
#ifndef _EXECUTE_BLEND_
const vec3 _BLEND_ZERO_ = vec3(0.0,0.0,0.0) ;
const vec3 _BLEND_QUARTER_ = vec3(0.25,0.25,0.25) ;
const vec3 _BLEND_HALF_ = vec3(0.5,0.5,0.5) ;
const vec3 _BLEND_ONE_  = vec3(1.0,1.0,1.0) ;

#define _EXECUTE_BLEND_(blendFunction, base, blend) vec4(((1.0-base.a)*blend.a*blend.rgb + (1.0-blend.a)*base.a*base.rgb + blend.a*base.a*blendFunction(blend.rgb, base.rgb, 1.0)) * (1.0 / (blend.a + (base.a * (1.0-blend.a)))), (blend.a + base.a*(1.0-blend.a)));

#endif


#if (defined(BLEND_ALL) || defined(BLEND_SOFTLIGHT)) && !defined(BLEND_SOFTLIGHT_)
vec3  D  (vec3 x) { return mix( sqrt(x), ((16.0*x-12.0)*x+4.0)*x  , vec3(lessThanEqual(x,_BLEND_QUARTER_))  ); }
#endif


#if (defined(BLEND_ALL) || defined(BLEND_VIVIDLIGHT)) && !defined(BLEND_VIVIDLIGHT_)
vec3 ocbrn(vec3 a, vec3 b, float f) {  return mix( _BLEND_ONE_ - min(_BLEND_ONE_,(_BLEND_ONE_-b)/(a*f+_BLEND_ONE_-f))    ,   _BLEND_ZERO_  , vec3(equal(a  ,_BLEND_ZERO_)) );  }
vec3 ocddg(vec3 a, vec3 b, float f) {  return mix(        min(_BLEND_ONE_,       b/(_BLEND_ONE_ - a*f))    ,   _BLEND_ONE_   , vec3(equal(a*f,_BLEND_ONE_ )) );  }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_DARKERCOLOR) || defined(BLEND_HUE) || defined(BLEND_SATURATION) || defined(BLEND_COLOR) || defined(BLEND_LUMINOSITY)) && !defined(BLEND_USE_LUM_)
#define BLEND_USE_LUM_


float lum(vec3 c) { return dot(c, vec3(0.3,0.59,0.11)); }
vec3 clipCol(vec3 c)
{
    vec3 o = c;  float l = lum(c);
    float n = min(c.r,min(c.g,c.b));
    float x = max(c.r,max(c.g,c.b));
    if(n<0.0) o = l + (o-l)*(l/(l-n));
    if(x>1.0) o = l + (o-l)*(1.0-l)/(x-l);
    return o;
}
vec3 setLum (vec3 c, float l) { return clipCol(c+l-lum(c)); }

#endif



#if (defined(BLEND_ALL) || defined(BLEND_HUE) || defined(BLEND_SATURATION)) && !defined(BLEND_USE_SAT_)
#define BLEND_USE_SAT_
float sat(vec3 c) { return max(c.x,max(c.y,c.z)) - min(c.x,min(c.y,c.z)); }
float midSat (vec3 v, float s) { return ((v.y-v.z)*s)/(v.x-v.z); }
vec3 setSat (vec3 c, float s)
{
    vec3 o;
    if(c.r==c.g && c.g==c.b) o = _BLEND_ZERO_;
    else if(c.r>c.g)  {
        if(c.r>c.b) {
            if(c.g>c.b)	o = vec3(s, midSat(c.rgb,s), 0.0);
            else	o = vec3(s, 0.0, midSat(c.rbg,s));
        }
        else		o = vec3(midSat(c.brg,s), 0.0, s);
    } else  {
        if(c.r<c.b) {
            if(c.g>c.b)	o = vec3(0.0, s, midSat(c.gbr,s));
            else	    o = vec3(0.0, midSat(c.bgr,s), s);
        }
        else		    o = vec3(midSat(c.grb,s), s, 0.0);
    }
    return o;
}
#endif




#if (defined(BLEND_ALL) || defined(BLEND_NORMAL)) && !defined(BLEND_NORMAL_)
#define BLEND_NORMAL_
vec3 blendNormal(vec3  a, vec3  b, float f) { return a; }
vec4 blendNormal(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendNormal, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_DISSOLVE)) && !defined(BLEND_DISSOLVE_)
#define BLEND_DISSOLVE_
vec3 blendDissolve(vec3  a, vec3  b, float f) { return a; }
vec4 blendDissolve(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendDissolve, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_DARKEN)) && !defined(BLEND_DARKEN_)
#define BLEND_DARKEN_
vec3 blendDarken(vec3  a, vec3  b, float f) { return min(a,b); }
vec4 blendDarken(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendDarken, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_MULTIPLY)) && !defined(BLEND_MULTIPLY_)
#define BLEND_MULTIPLY_
vec3 blendMultiply(vec3  a, vec3  b, float f) { return a*b; }
vec4 blendMultiply(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendMultiply, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_COLORBURN)) && !defined(BLEND_COLORBURN_)
#define BLEND_COLORBURN_
vec3 blendColorBurn(vec3  a, vec3  b, float f) {   vec3 d = (a*f+_BLEND_ONE_-vec3(f));  return mix(mix(_BLEND_ONE_-((_BLEND_ONE_-b)/d), _BLEND_ZERO_, vec3(greaterThanEqual(_BLEND_ONE_-b,d)) ), _BLEND_ONE_   ,   vec3(equal(b,_BLEND_ONE_) ));    }
vec4 blendColorBurn(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendColorBurn, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_LINEARBURN)) && !defined(BLEND_LINEARBURN_)
#define BLEND_LINEARBURN_
vec3 blendLinearBurn(vec3  a, vec3  b, float f) { return max(_BLEND_ZERO_, a*f+b-f); }
vec4 blendLinearBurn(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendLinearBurn, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_DARKERCOLOR)) && !defined(BLEND_DARKERCOLOR_)
#define BLEND_DARKERCOLOR_
vec3 blendDarkerColor(vec3  a, vec3  b, float f) { return ( lum(a)<lum(b) ? a : b ); }
vec4 blendDarkerColor(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendDarkerColor, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_LIGHTEN)) && !defined(BLEND_LIGHTEN_)
#define BLEND_LIGHTEN_
vec3 blendLighten(vec3  a, vec3  b, float f) { return max(a,b); }
vec4 blendLighten(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendLighten, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_SCREEN)) && !defined(BLEND_SCREEN_)
#define BLEND_SCREEN_
vec3 blendScreen(vec3  a, vec3  b, float f) { return b+a-b*a; }
vec4 blendScreen(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendScreen, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_COLORDODGE)) && !defined(BLEND_COLORDODGE_)
#define BLEND_COLORDODGE_
vec3 blendColorDodge(vec3  a, vec3  b, float f) { a*=f;  return mix(    mix(  min(_BLEND_ONE_,b/(1.0-a)), _BLEND_ONE_,  step(1.0-a, b))     , _BLEND_ZERO_, vec3(equal(b,_BLEND_ZERO_)) ); }
vec4 blendColorDodge(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendColorDodge, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_LINEARDODGE)) && !defined(BLEND_LINEARDODGE_)
#define BLEND_LINEARDODGE_
vec3 blendLinearDodge(vec3  a, vec3  b, float f) { a*=f;  return min(_BLEND_ONE_,a+b); }
vec4 blendLinearDodge(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendLinearDodge, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_LIGHTERCOLOR)) && !defined(BLEND_LIGHTERCOLOR_)
#define BLEND_LIGHTERCOLOR_
vec3 blendLighterColor(vec3  a, vec3  b, float f) { return ( lum(a)>lum(b) ? a : b ); }
vec4 blendLighterColor(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendLighterColor, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_OVERLAY)) && !defined(BLEND_OVERLAY_)
#define BLEND_OVERLAY_
vec3 blendOverlay(vec3  a, vec3  b, float f) { return mix( a+(2.0*b -1.0)-a*(2.0*b-1.0) ,  2.0*b*a , step(-_BLEND_HALF_,-b) ); }
vec4 blendOverlay(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendOverlay, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_SOFTLIGHT)) && !defined(BLEND_SOFTLIGHT_)
#define BLEND_SOFTLIGHT_
vec3 blendSoftLight(vec3  a, vec3  b, float f) { return mix( b+(2.0*a -1.0)*(D(b)-b)  , b-(1.0-2.0*a)*b*(1.0-b) , step(-_BLEND_HALF_,-a) ); }
vec4 blendSoftLight(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendSoftLight, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_HARDLIGHT)) && !defined(BLEND_HARDLIGHT_)
#define BLEND_HARDLIGHT_
vec3 blendHardLight(vec3  a, vec3  b, float f) { return mix( b+(2.0*a -1.0)-b*(2.0*a-1.0) ,  2.0*a*b , step(-_BLEND_HALF_,-a) ); }
vec4 blendHardLight(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendHardLight, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_VIVIDLIGHT)) && !defined(BLEND_VIVIDLIGHT_)
#define BLEND_VIVIDLIGHT_
vec3 blendVividLight(vec3  a, vec3  b, float f) { return mix( ocddg(2.0*a-1.0,b,f)  ,  ocbrn(2.0*a,b,f)  ,  vec3(greaterThanEqual(_BLEND_HALF_,a)));  }
vec4 blendVividLight(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendVividLight, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_LINEARLIGHT)) && !defined(BLEND_LINEARLIGHT_)
#define BLEND_LINEARLIGHT_
vec3 blendLinearLight(vec3  a, vec3  b, float f) { return mix( min(_BLEND_ONE_, (2.0*a-1.0)*f+b) , max(_BLEND_ZERO_, 2.0*a*f+b-1.0)  , step(-_BLEND_HALF_,-a)); }
vec4 blendLinearLight(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendLinearLight, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_PINLIGHT)) && !defined(BLEND_PINLIGHT_)
#define BLEND_PINLIGHT_
vec3 blendPinLight(vec3  a, vec3  b, float f) { return mix( max(2.0*a-1.0,b) , min(2.0*a, b) , step(-_BLEND_HALF_,-a)  ); }
vec4 blendPinLight(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendPinLight, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_HARDMIX)) && !defined(BLEND_HARDMIX_)
#define BLEND_HARDMIX_
vec3 blendHardMix(vec3  a, vec3  b, float f) { return  min( _BLEND_ONE_, max(_BLEND_ZERO_,  (b+a*f-f)/(1.0-f+1e-6)  ))  ;   }
vec4 blendHardMix(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendHardMix, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_DIFFERENCE)) && !defined(BLEND_DIFFERENCE_)
#define BLEND_DIFFERENCE_
vec3 blendDifference(vec3  a, vec3  b, float f) { return abs(a*f-b); }
vec4 blendDifference(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendDifference, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_EXCLUSION)) && !defined(BLEND_EXCLUSION_)
#define BLEND_EXCLUSION_
vec3 blendExclusion(vec3  a, vec3  b, float f) { return a+b-2.0*a*b; }
vec4 blendExclusion(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendExclusion, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_SUBTRACT)) && !defined(BLEND_SUBTRACT_)
#define BLEND_SUBTRACT_
vec3 blendSubtract(vec3  a, vec3  b, float f) { return max(b-a, _BLEND_ZERO_); }
vec4 blendSubtract(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendSubtract, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_DIVIDE)) && !defined(BLEND_DIVIDE_)
#define BLEND_DIVIDE_
vec3 blendDivide(vec3  a, vec3  b, float f) { return min(b/a, _BLEND_ONE_); }
vec4 blendDivide(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendDivide, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_HUE)) && !defined(BLEND_HUE_)
#define BLEND_HUE_
vec3 blendHue(vec3  a, vec3  b, float f) { return  setLum( setSat(a, sat(b)) , lum(b) );  }
vec4 blendHue(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendHue, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_SATURATION)) && !defined(BLEND_SATURATION_)
#define BLEND_SATURATION_
vec3 blendSaturation(vec3  a, vec3  b, float f) { return  setLum( setSat(b, sat(a)) , lum(b) );  }
vec4 blendSaturation(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendSaturation, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_COLOR)) && !defined(BLEND_COLOR_)
#define BLEND_COLOR_
vec3 blendColor(vec3  a, vec3  b, float f) { return  setLum( a, lum(b) );  }
vec4 blendColor(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendColor, base, blend ); }
#endif

#if (defined(BLEND_ALL) || defined(BLEND_LUMINOSITY)) && !defined(BLEND_LUMINOSITY_)
#define BLEND_LUMINOSITY_
vec3 blendLuminosity(vec3  a, vec3  b, float f) { return  setLum( b, lum(a) );  }
vec4 blendLuminosity(vec4 base, vec4 blend) { return _EXECUTE_BLEND_(blendLuminosity, base, blend ); }
#endif

