#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

#define MAXPOINTS 20

///////////////////////////////////////////////////////////
// Uniforms
uniform sampler2D u_texture;
varying vec2 v_texCoord;
varying vec2 v_originTexCoord;
uniform float aspectRatio;


uniform vec2 center[MAXPOINTS];
uniform vec2 radius[MAXPOINTS];
uniform float scale[MAXPOINTS];
uniform float angle[MAXPOINTS];
uniform float u_min[MAXPOINTS];
uniform float u_max[MAXPOINTS];


uniform float types[MAXPOINTS];
uniform float count;



void main()
{
    
    
    vec2 uv = v_originTexCoord;
    
    
    gl_FragColor = texture2D(u_texture, v_texCoord);

    int cnt = int(count);
    for (int i = 0; i < cnt; i++)
    {
        
        
        if(types[i] == 0.0) continue;
        
        
        vec2 textureCoordinateToUse = uv;
        
        float e1 =  ( textureCoordinateToUse.x - center[i].x ) / ( radius[i].x );
        float e2 =  ( textureCoordinateToUse.y - center[i].y ) / ( radius[i].y / aspectRatio );
        float d  =  (e1 * e1) + (e2 * e2);
        
      
        if (d < 1.0)
        {
            
          
            
            if(types[i] == 1.0) {
                
                // bulge
                vec2 dist = vec2(d * radius[i].x, d * radius[i].y);
                
                vec2 delta = ((radius[i] - dist) / radius[i]);
                float deltaScale = scale[i];
                if(deltaScale > 0.0) {
                    deltaScale = smoothstep(u_min[i], u_max[i], deltaScale);
                }
                
                vec2 percent = 1.0 - (delta * deltaScale);
                
                gl_FragColor = mix(gl_FragColor, vec4(1.0, 0, 0, 1.0), 1.0 - percent.x);

                
                
            } else {
                
                // shift
                float dist = 1.0 - d;
                float delta = scale[i] * dist;
                float deltaScale = smoothstep(u_min[i], u_max[i], dist);
            
                gl_FragColor = mix(gl_FragColor, vec4(1.0, 0, 0, 1.0), deltaScale);
                
            }
            
            
            
            
            
            // out line
            if(d > 0.95) {
                gl_FragColor = mix(gl_FragColor, vec4(1.0, 0, 0, 1.0), 1.0);
                return;
            }
            

            if(d < 0.0025) gl_FragColor = mix(gl_FragColor, vec4(0.0, 1.0, 1.0, 1.0), 1.0);
            
            
        }
        
    
        
        
    }
    
    //gl_FragColor = vec4(v_texCoord.x, v_texCoord.y, 0, 1.0);

    
    
}
