#define MAXPOINTS 20

attribute vec2 a_texCoord;
varying vec2 v_texCoord;

uniform float aspectRatio;

uniform vec2 center[MAXPOINTS];
uniform vec2 radius[MAXPOINTS];
uniform float scale[MAXPOINTS];
uniform float angle[MAXPOINTS];
uniform float u_min[MAXPOINTS];
uniform float u_max[MAXPOINTS];
uniform float types[MAXPOINTS];

uniform int count;
uniform float progress;
uniform float faceYFactor;

vec2 distortion(vec2 uv, int i) {

    vec2 textureCoordinateToUse = uv;

    float e1 =  ( textureCoordinateToUse.x - center[i].x ) / ( radius[i].x );
    float e2 =  ( textureCoordinateToUse.y - center[i].y ) / ( radius[i].y / aspectRatio );
    float d  = (e1 * e1) + (e2 * e2);

    if (d < 1.0) {
        if(types[i] == 1.0) {
            
            // bulge
            vec2 dist = vec2(d * radius[i].x, d * radius[i].y);
            textureCoordinateToUse -= center[i];

            vec2 delta = ((radius[i] - dist) / radius[i]);
            float deltaScale = scale[i];
            if(deltaScale > 0.0) {
                deltaScale = smoothstep(u_min[i], u_max[i], deltaScale);
            }

            vec2 percent = 1.0 - ((delta * deltaScale) * progress);
            textureCoordinateToUse = textureCoordinateToUse * percent;
            uv = textureCoordinateToUse + center[i];
            
            
        } else if(types[i] == 2.0) {
            // shift
            float dist = 1.0 - d;
            float delta = scale[i] * dist * progress;

            float deltaScale = smoothstep(u_min[i], u_max[i], dist);
            float directionX = cos(angle[i]) * deltaScale;
            float directionY = sin(angle[i]) * deltaScale / (faceYFactor * aspectRatio);

            uv = vec2(textureCoordinateToUse.x - (delta * directionX), textureCoordinateToUse.y -  (delta * directionY));
        }
    }

    return uv;
}

void main() {
    vec2 uv = a_texCoord;

    gl_Position = vec4(uv * 2.0 - 1.0, 0.0, 1.0);

    for (int i = 0; i < count; i++) {
        uv = distortion(uv, i);
    }

    v_texCoord = uv;
}