#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

#define SAMPLES 14

///////////////////////////////////////////////////////////
// Uniforms
uniform sampler2D u_texture;
uniform sampler2D prev_texture;
uniform lowp float threshold;
uniform lowp float intensity;

///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;
varying vec2 v_sampleCoordinates[SAMPLES];

#include "RGB_TO_HSV.frag"

void main()
{
   lowp float maxValue = 0.0;
   for (int i = 0; i < SAMPLES; i++) {
       lowp vec4 color = texture2D(u_texture, v_sampleCoordinates[i]);
       lowp vec3 hsv = rgb2hsv(color.rgb);
       maxValue = max(maxValue, (1.0 - hsv.g) * hsv.b);
   }
   lowp vec4 textureColor = texture2D(u_texture, v_texCoord);
   lowp vec4 prevValue = texture2D(prev_texture, v_texCoord);
   lowp vec3 hsv_textureColor = rgb2hsv(textureColor.rgb);
   float textureValue = (1.0 - hsv_textureColor.g) * hsv_textureColor.b;
   maxValue = maxValue * 1.03;
   float minY = 0.05;
   float maxY = 1.0;
   lowp float value = pow(2.71828, log(minY / maxY) / (maxValue * (threshold - 1.0)) * textureValue + log(maxY) - log(minY / maxY)/ (threshold - 1.0));
   value = clamp(value, 0.0, maxY);
   value = pow(value, 0.8);
   float validValue = 0.6 * value + 0.3 * prevValue.r + 0.1 * prevValue.g;
   prevValue.g = prevValue.r;
   prevValue.r = value;
   gl_FragColor = vec4(validValue * intensity, prevValue.rgb);
}

