uniform float u_contrast;
uniform float u_gamma_adjust;

uniform vec4  u_outline_color;  // new
uniform float u_outline_thickness;
uniform float u_boldness;
uniform vec4  u_color;
varying vec4 v_glyph;


#define SQRT2_2 0.70710678118654757 /* 1 / sqrt(2.) */
#define SQRT2   1.4142135623730951

struct glyph_info_t {
    ivec2 nominal_size;
    ivec2 atlas_pos;
};

glyph_info_t glyph_info_decode (vec4 v)
{
    glyph_info_t gi;
    gi.nominal_size = (ivec2 (mod (v.zw, 256.)) + 2) / 4;
    gi.atlas_pos = ivec2 (v_glyph.zw) / 256;
    return gi;
}


float antialias (float d)
{
    return smoothstep (-.75, +.75, d);
}

void main()
{
    vec2 p = v_glyph.xy;
    glyph_info_t gi = glyph_info_decode (v_glyph);
    
    /* isotropic antialiasing */
    vec2 dpdx = dFdx (p);
    vec2 dpdy = dFdy (p);
    float m = length (vec2 (length (dpdx), length (dpdy))) * SQRT2_2;
    
    vec4 color = u_color;
    
    float gsdist = glyphy_sdf (p, gi.nominal_size GLYPHY_DEMO_EXTRA_ARGS);
    float sdist = gsdist / m * u_contrast;
    
    sdist -= u_boldness * 10.;
    
    float odist = sdist;
    float fillDist = sdist;

    bool  u_outline = u_outline_thickness > 0.0;
    
    vec4 outlineColor = u_outline_color;
    
    if (u_outline) {

        odist = abs (odist) - u_outline_thickness * 0.5;
        fillDist += u_outline_thickness * 0.5;
        
        if (odist <= 1.) {
            float alpha = antialias (-odist);
            if (u_gamma_adjust != 1.)
                alpha = pow (alpha, 1./u_gamma_adjust);
            outlineColor = vec4 (outlineColor.rgb,outlineColor.a * alpha);
        } else {
            outlineColor = vec4(0.0);
        }

        
    }
    
    
    // fill
    if (odist > 1. && fillDist > 1.)
        discard;

    if(fillDist <= 1.) {
        // anti aliasing
        float alpha = smoothstep (-.75, +.75, -fillDist);
        if (u_gamma_adjust != 1.)
            alpha = pow (alpha, 1./u_gamma_adjust);
        color = vec4 (color.rgb,color.a * alpha);

    } else {
        color = vec4(1.0, 1.0, 1.0, 0.0);
    }
    
    color = vec4(color.rgb * (1.0 - outlineColor.a) + outlineColor.rgb * outlineColor.a, color.a * (1.0 - outlineColor.a) + outlineColor.a);
    
        
#ifdef DEBUG
    
    
    color = vec4 (0,0,0,0);
    
    // Color the inside of the glyph a light red
    color += vec4 (.5,0,0,.5) * smoothstep (1., -1., sdist);
    
    float udist = abs (sdist);
    float gudist = abs (gsdist);
    // Color the outline red
    color += vec4 (1,0,0,1) * smoothstep (2., 1., udist);
    // Color the distance field in green
    if (!glyphy_isinf (udist))
        color += vec4(0,.4,0,.4 - (abs(gsdist) / max(float(gi.nominal_size.x), float(gi.nominal_size.y))) * 4.);
    
    float pdist = glyphy_point_dist (p, gi.nominal_size GLYPHY_DEMO_EXTRA_ARGS);
    // Color points green
    color = mix (vec4 (0,1,0,.5), color, smoothstep (.05, .06, pdist));
    
    glyphy_arc_list_t arc_list = glyphy_arc_list (p, gi.nominal_size GLYPHY_DEMO_EXTRA_ARGS);
    // Color the number of endpoints per cell blue
    color += vec4 (0,0,1,.1) * float(arc_list.num_endpoints) * 32./255.;
    
    
#endif
    
    gl_FragColor = color;
}
