#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

varying vec2        v_texCoord;
uniform vec2        iResolution;
uniform sampler2D   iChannel0;
uniform int         iKernelSize;

float vec3_float_flat( in vec3 t )
{
    const vec3 coeffs = vec3( 1.0/3.0, 1.0/3.0, 1.0/3.0 );
    t *= coeffs;
    return t.r + t.g + t.b;
}

void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
    
    float kernel_sq = float( iKernelSize * iKernelSize);
    
    fragColor = vec4(0.0,0.0,1.0,1.0);
    
    vec3 means = vec3(0.0);
    vec3 deviations = vec3(0.0);

    for( int i = 0; i < iKernelSize; i++ )
    {
        for( int j = 0; j < iKernelSize; j++ )
        {
            vec2 uv = fragCoord;
            uv += vec2( i,j);
            uv += vec2( -1.0, -1.0 );
            uv /= iResolution.xy;
            vec3 sample3 = texture2D( iChannel0, uv ).rgb;
            means += sample3 / kernel_sq;

            deviations += sample3*sample3 / kernel_sq;
        }
    }
    deviations -= ( means * means);

    
    
    float deviation = vec3_float_flat( deviations );
    fragColor.a = deviation;
    
    fragColor.rgb = means;
    //deviations *= 10.0;
    //fragColor.rgb = deviations;

}

void main( void ) {
    vec4 color = vec4(0.0);

    vec2 bgTexCoord = gl_FragCoord.xy;
    //http://bts.snowcorp.com/browse/SELFIECAM-308
    if(bgTexCoord.y < 0.0){
        bgTexCoord.y = iResolution.y + bgTexCoord.y;
    }

    mainImage( color, bgTexCoord );
    gl_FragColor = color;
}