#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

///////////////////////////////////////////////////////////
// Uniforms
uniform sampler2D u_texture;
uniform sampler2D u_filter_texture;
uniform sampler2D u_mask_texture;
uniform sampler2D u_glow_lip_texture;
uniform sampler2D u_glow_mask_texture;
uniform float intensity;

///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;
varying vec2 v_texCoord1;


void main()
{
    vec4 textureColor = texture2D(u_texture, v_texCoord);
    vec4 maskTextureColor = texture2D(u_mask_texture, v_texCoord1);
    vec4 glowTextureColor = texture2D(u_glow_lip_texture, v_texCoord);
    vec4 glowMaskTextureColor = texture2D(u_glow_mask_texture, v_texCoord1);

    float blueColor = textureColor.b * 63.0;
    vec2 quad1 = vec2(0.0,0.0);
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);

    vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);

    vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos1.y = (quad1.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);

    vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos2.y = (quad2.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);

    texPos1.y = 1.0 - texPos1.y;
    texPos2.y = 1.0 - texPos2.y;
    
    vec4 newColor1 = texture2D(u_filter_texture, texPos1);
    vec4 newColor2 = texture2D(u_filter_texture, texPos2);

    vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
    vec4 lipColor = mix(textureColor, vec4(newColor.rgb, textureColor.w), intensity*maskTextureColor.r);
    float glowMask = mix(glowTextureColor.r, 0.0, 1.0 - glowMaskTextureColor.r);
	gl_FragColor = vec4(mix(lipColor.rgb, vec3(1.0), glowTextureColor.r), lipColor.a);
}


