precision highp float;

uniform sampler2D uTexture;
uniform sampler2D uAlphaMaskTexture;

uniform float uDiscardAlphaValue;
uniform float uMaskAlphaThreshold;

uniform vec4 uEdgeColor;
uniform vec4 uOuterEdgeColor;

varying vec2 v_texCoord;

const float edgeThreshold = 0.1;

void main() {
    vec4 maskColor = texture2D(uAlphaMaskTexture, v_texCoord);
    maskColor.a = abs(uDiscardAlphaValue - maskColor.a);

    if (maskColor.r > edgeThreshold) {
        //Inner edge area
        gl_FragColor = uEdgeColor;
    } else if (maskColor.g > edgeThreshold) {
        //Inter space area
    } else if (maskColor.b > edgeThreshold) {
        //Outer edge area
        gl_FragColor = uOuterEdgeColor;
    } else if (maskColor.a > uMaskAlphaThreshold) {
        //Segmented area
        gl_FragColor = vec4(texture2D(uTexture, v_texCoord).rgb, 1.0);
    }
}