#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

varying vec2 textureCoordinate;
varying vec2 textureCoordinateUpscale;
varying vec2 textureScale;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform float blurOpacity;
uniform float factor1;
uniform float factor2;
uniform float factor3;
uniform float factor4;
uniform float factor5;
uniform float factor6;
uniform float factor7;
uniform float filterOpacity;

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    vec3 hsv = vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
    return hsv;
}

vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con)
{
    const float AvgLumR = 0.5;
    const float AvgLumG = 0.5;
    const float AvgLumB = 0.5;
    
    const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
    
    vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
    vec3 brtColor = color * brt;
    vec3 intensity = vec3(dot(brtColor, LumCoeff));
    vec3 satColor = mix(intensity, brtColor, sat);
    vec3 conColor = mix(AvgLumin, satColor, con);
    
    return conColor;
}

void main() {
    vec4 inputColor = texture2D(inputImageTexture, textureCoordinate);
    
    vec3 hsv = rgb2hsv(inputColor.rgb);
    
    lowp float opacityLimit = 1.0;
    if ((0.18 <= hsv.x && hsv.x <= 0.89) || hsv.z <= 0.2)
    {
        opacityLimit = 0.0;
    }
    
    if (0.16 < hsv.x && hsv.x < 0.18)
    {
        opacityLimit = min(opacityLimit, (0.18 - hsv.x) / 0.02);
    }
    
    if (0.89 < hsv.x && hsv.x < 0.91)
    {
        opacityLimit = min(opacityLimit, 1.0 - (0.91 - hsv.x) / 0.02);
    }
    
    if (0.2 < hsv.z && hsv.x < 0.3)
    {
        opacityLimit = min(opacityLimit, 1.0 - (0.3 - hsv.z) / 0.1);
    }
    
    if (opacityLimit == 0.0)
    {
        gl_FragColor = inputColor;
        return;
    }
    
    lowp vec4 blurColor = texture2D(inputImageTexture2, textureCoordinate);
    
    opacityLimit = blurOpacity * opacityLimit;
    
    lowp float cDistance = distance(vec3(0.0, 0.0, 0.0), max(blurColor.rgb - inputColor.rgb, 0.0)) * factor1;
    lowp vec3 brightColor = ContrastSaturationBrightness(inputColor.rgb, factor2, 1.0, factor3);
    lowp vec3 mix11Color = mix(inputColor.rgb, brightColor.rgb, cDistance);
    
    lowp float dDistance = distance(vec3(0.0, 0.0, 0.0), max(inputColor.rgb-blurColor.rgb, 0.0)) * factor4;
    lowp vec3 darkColor = ContrastSaturationBrightness(inputColor.rgb, factor5, 1.0, factor6);
    lowp vec3 mix115Color = mix(mix11Color.rgb, darkColor.rgb, dDistance);
    
    lowp vec3 mix12Color;
    
    if (factor7 < 0.999)
    {
        lowp vec3 mix116Color = mix(inputColor.rgb, mix115Color.rgb, factor7);
        mix12Color = mix(mix116Color.rgb, blurColor.rgb, opacityLimit);
    }
    else
    {
        mix12Color = mix(mix115Color.rgb, blurColor.rgb, opacityLimit);
    }
    
    if (filterOpacity < 0.999)
    {
        gl_FragColor = vec4(mix(inputColor.rgb, mix12Color.rgb, filterOpacity), 1.0);
    }
    else
    {
        gl_FragColor = vec4(mix12Color.rgb, 1.0);
    }
    
    
}
