#ifndef MAX_DISTORTION_COUNT
#define MAX_DISTORTION_COUNT 30
#endif

attribute vec4 a_position;
attribute vec4 a_texCoord;
varying vec2 v_texCoord;
uniform vec2 uDistortionConditions;
uniform vec3 uItemTypePoint[MAX_DISTORTION_COUNT];
uniform vec4 uItemAffectRange[MAX_DISTORTION_COUNT];
uniform vec4 uItemPrecalcCoefficientData[MAX_DISTORTION_COUNT];
uniform vec4 uItemPrecalcAngleData[MAX_DISTORTION_COUNT];

void getCircularZoomedPosition(inout vec4 currentPosition, in int itemIndex){
    float dist = distance( vec2(currentPosition.x, currentPosition.y * uDistortionConditions.x), vec2(uItemTypePoint[itemIndex].y, uItemTypePoint[itemIndex].z * uDistortionConditions.x));
    if(dist < uItemAffectRange[itemIndex].x){
        vec2 positionDelta = uItemTypePoint[itemIndex].yz - currentPosition.xy;
        float distRatio = dist / uItemAffectRange[itemIndex].x;
        // s3 에서 float weight = uItemPrecalcCoefficientData[itemIndex].x * (uItemPrecalcCoefficientData[itemIndex].w + cos(distRatio * uItemPrecalcCoefficientData[itemIndex].y + uItemPrecalcCoefficientData[itemIndex].z)); 는 좌표 깨짐.
        float cosdist = cos(distRatio * uItemPrecalcCoefficientData[itemIndex].y + uItemPrecalcCoefficientData[itemIndex].z);
        float midWeight = (uItemPrecalcCoefficientData[itemIndex].w + cosdist);
        float weight = uItemPrecalcCoefficientData[itemIndex].x * midWeight;
        currentPosition.xy -= positionDelta * weight;
    }
}

void getCircularDirectionedPosition(inout vec4 currentPosition, in int itemIndex){
    float dist = distance( vec2(currentPosition.x, currentPosition.y * uDistortionConditions.x), vec2(uItemTypePoint[itemIndex].y, uItemTypePoint[itemIndex].z * uDistortionConditions.x));
    if(dist < uItemAffectRange[itemIndex].x){
        float distRatio = dist / uItemAffectRange[itemIndex].x;
        float cosdist = cos(distRatio * uItemPrecalcCoefficientData[itemIndex].y);
        float weight = uItemPrecalcCoefficientData[itemIndex].x * cosdist;
        vec2 directionVector = uItemAffectRange[itemIndex].zw * weight;
        currentPosition.x += (directionVector.x * uItemPrecalcAngleData[itemIndex].y - directionVector.y * uItemPrecalcAngleData[itemIndex].x);
        currentPosition.y += (directionVector.y * uItemPrecalcAngleData[itemIndex].y + directionVector.x * uItemPrecalcAngleData[itemIndex].x) / uDistortionConditions.x;
    }
}

float getEllipseRange(in vec4 currentPosition, in int itemIndex){
    vec2 deltaVector = vec2(currentPosition.x - uItemTypePoint[itemIndex].y, (currentPosition.y - uItemTypePoint[itemIndex].z) *  uDistortionConditions.x);
    float x = uItemPrecalcAngleData[itemIndex].w * deltaVector.x + uItemPrecalcAngleData[itemIndex].z * deltaVector.y;
    float y = uItemPrecalcAngleData[itemIndex].z * deltaVector.x - uItemPrecalcAngleData[itemIndex].w * deltaVector.y;
    return pow(abs(x), 2.0) / pow(uItemAffectRange[itemIndex].y, 2.0) + pow(abs(y), 2.0) / pow(uItemAffectRange[itemIndex].x, 2.0);
}

void getEllipseZoomedPosition(inout vec4 currentPosition, in int itemIndex){
    float ellipseRange = getEllipseRange(currentPosition, itemIndex);
    if(ellipseRange < 1.0){
        vec2 positionDelta = uItemTypePoint[itemIndex].yz - currentPosition.xy;
        float cosEllipseRange = cos(sqrt(ellipseRange) * uItemPrecalcCoefficientData[itemIndex].y + uItemPrecalcCoefficientData[itemIndex].z);
        float weight = uItemPrecalcCoefficientData[itemIndex].x * ( uItemPrecalcCoefficientData[itemIndex].w  +  cosEllipseRange);
        currentPosition.xy -= positionDelta * weight;
    }
}

void getEllipseDirectionedPosition(inout vec4 currentPosition, in int itemIndex){
    float ellipseRange = getEllipseRange(currentPosition, itemIndex);
    if(ellipseRange < 1.0){
        float weight = uItemPrecalcCoefficientData[itemIndex].x * cos( uItemPrecalcCoefficientData[itemIndex].y * sqrt(ellipseRange));
        vec2 directionVector = uItemAffectRange[itemIndex].zw * weight;
        currentPosition.x += (directionVector.x * uItemPrecalcAngleData[itemIndex].y - directionVector.y * uItemPrecalcAngleData[itemIndex].x);
        currentPosition.y += (directionVector.y * uItemPrecalcAngleData[itemIndex].y + directionVector.x * uItemPrecalcAngleData[itemIndex].x) / uDistortionConditions.x;
    }
}

void getUniformZoomInedPosition(inout vec4 currentPosition, in int itemIndex){
    float range = getEllipseRange(currentPosition, itemIndex);
    if(range < 1.0){
        vec2 delta = uItemTypePoint[itemIndex].yz - currentPosition.xy;
        currentPosition.xy = uItemTypePoint[itemIndex].yz - (delta * uItemPrecalcCoefficientData[itemIndex].x);
        float distRatio = getEllipseRange(currentPosition, itemIndex);
        if(distRatio >= 1.0){
            float originLength = distance(delta, vec2(0.0, 0.0));
            vec2 directionVector = delta / originLength;
            float constantWeight = originLength / sqrt(range);
            currentPosition.xy = uItemTypePoint[itemIndex].yz - (directionVector * constantWeight);
        }
    }
}

void getUniformZoomInedPositionWithAdjustment(inout vec4 currentPosition, in int itemIndex){
    float maxZoomRange = pow(uItemPrecalcCoefficientData[itemIndex].x, 2.0);
    float range = getEllipseRange(currentPosition, itemIndex);
    if (range <= maxZoomRange) {
        vec2 delta = uItemTypePoint[itemIndex].yz - currentPosition.xy;
        currentPosition.xy = uItemTypePoint[itemIndex].yz - (delta * uItemPrecalcCoefficientData[itemIndex].x);
        float zoomedRange = getEllipseRange(currentPosition, itemIndex);
        if (zoomedRange > 1.0) { // 확대된 영역 밖의 영역
            float deltaScala = distance(delta, vec2(0.0, 0.0));
            vec2 directionVector = delta / deltaScala;
            float minAdjustRange = 1.0 / maxZoomRange;
            float deltaRangeRatio = (range - minAdjustRange)  / (maxZoomRange - minAdjustRange);
            float originalLength = deltaScala / sqrt(range);
            currentPosition.xy = uItemTypePoint[itemIndex].yz - (directionVector * (originalLength + originalLength * (sqrt(maxZoomRange) - 1.0) * deltaRangeRatio));
        }
    }
}

void getUniformZoomOutedPosition(inout vec4 currentPosition, in int itemIndex){
    float range = getEllipseRange(currentPosition, itemIndex);
    float minZoomRange = pow(uItemPrecalcCoefficientData[itemIndex].x, 2.0);

    if (range < 1.0) {
        vec2 delta = uItemTypePoint[itemIndex].yz - currentPosition.xy;
        if(range < minZoomRange){
            currentPosition.xy = uItemTypePoint[itemIndex].yz - (delta * uItemPrecalcCoefficientData[itemIndex].x);
        }else{
            float deltaScala = distance(delta, vec2(0.0, 0.0));
            vec2 directionVector = delta / deltaScala;
            float deltaRangeRatio = 1.0 - ((range - minZoomRange) / (1.0 - minZoomRange));
            float originalLength = deltaScala / sqrt(range);
            currentPosition.xy = uItemTypePoint[itemIndex].yz - (directionVector * (originalLength - originalLength * (1.0 - minZoomRange) * deltaRangeRatio));
        }
    }
}


void main()
{
   vec4 newPosition = a_position;
   if (a_position.x > -1.0 && a_position.x < 1.0 && a_position.y > -1.0 && a_position.y < 1.0) {
      for (int itemIndex = 0; itemIndex < int(uDistortionConditions.y); itemIndex++) {
          float type = uItemTypePoint[itemIndex].x;
          if (type ==  1.0 || type ==  2.0){
            getCircularZoomedPosition(newPosition, itemIndex);
          }else if (type ==  3.0) {
            getCircularDirectionedPosition(newPosition, itemIndex);
          }else if (type ==  4.0 || type ==  5.0) {
            getEllipseZoomedPosition(newPosition, itemIndex);
          }else if (type ==  6.0) {
            getEllipseDirectionedPosition(newPosition, itemIndex);
          }else if (type ==  8.0) {
            getUniformZoomInedPositionWithAdjustment(newPosition, itemIndex);
          }else if (type ==  7.0) {
            getUniformZoomInedPosition(newPosition, itemIndex);
          }else if (type ==  9.0) {
            getUniformZoomOutedPosition(newPosition, itemIndex);
          }
       }
   }
   gl_Position = vec4(newPosition.xy, 0.0, 1.0);
   v_texCoord = a_texCoord.xy;
}