#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

#ifndef DIRECTIONAL_LIGHT_COUNT
#define DIRECTIONAL_LIGHT_COUNT 0
#endif
#ifndef SPOT_LIGHT_COUNT
#define SPOT_LIGHT_COUNT 0
#endif
#ifndef POINT_LIGHT_COUNT
#define POINT_LIGHT_COUNT 0
#endif
#if (DIRECTIONAL_LIGHT_COUNT > 0) || (POINT_LIGHT_COUNT > 0) || (SPOT_LIGHT_COUNT > 0)
#define LIGHTING
#endif


#ifdef REFLECT
#define LIGHTING
#define SPECULAR
#endif


///////////////////////////////////////////////////////////
// Uniforms
uniform vec3 u_ambientColor;

uniform sampler2D u_diffuseTexture;

#if defined(LIGHTING) || defined(CUBE_MAP)
uniform mat4 u_deviceRoll;
uniform mat4 u_deviceInvRoll;
#endif

#if defined(LIGHTMAP)
uniform sampler2D u_lightmapTexture;
#endif

#if defined(LIGHTING)
varying mat3 v_WorldViewM;
#if defined(BUMPED)
uniform sampler2D u_normalmapTexture;
uniform float u_normalMixFactor;
#endif

#if defined(ROUGHNESS)
uniform sampler2D u_roughnessTexture;
#endif

#if (DIRECTIONAL_LIGHT_COUNT > 0)
uniform vec3 u_directionalLightColor[DIRECTIONAL_LIGHT_COUNT];
//#if !defined(BUMPED)
uniform vec3 u_directionalLightDirection[DIRECTIONAL_LIGHT_COUNT];
//#endif
#endif

#if (POINT_LIGHT_COUNT > 0)
uniform vec3 u_pointLightColor[POINT_LIGHT_COUNT];
uniform vec3 u_pointLightPosition[POINT_LIGHT_COUNT];
uniform float u_pointLightRangeInverse[POINT_LIGHT_COUNT];
#endif

#if (SPOT_LIGHT_COUNT > 0)
uniform vec3 u_spotLightColor[SPOT_LIGHT_COUNT];
uniform float u_spotLightRangeInverse[SPOT_LIGHT_COUNT];
uniform float u_spotLightInnerAngleCos[SPOT_LIGHT_COUNT];
uniform float u_spotLightOuterAngleCos[SPOT_LIGHT_COUNT];
#if !defined(BUMPED)
uniform vec3 u_spotLightDirection[SPOT_LIGHT_COUNT];
#endif
#endif

#if defined(SPECULAR)
uniform float u_specularExponent;
#endif

#endif

#if defined(MODULATE_COLOR)
uniform vec4 u_modulateColor;
#endif

#if defined(MODULATE_ALPHA)
uniform float u_modulateAlpha;
#endif

///////////////////////////////////////////////////////////
// Variables
vec4 _baseColor;

///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;

#if defined(LIGHTMAP)
varying vec2 v_texCoord1;
#endif

#if defined(LIGHTING) || defined(CUBE_MAP)

//#if !defined(BUMPED)
varying vec3 v_normalVector;
//#endif

//#if defined(BUMPED) && (DIRECTIONAL_LIGHT_COUNT > 0)
//varying vec3 v_directionalLightDirection[DIRECTIONAL_LIGHT_COUNT];
//#endif

#if (POINT_LIGHT_COUNT > 0)
varying vec3 v_vertexToPointLightDirection[POINT_LIGHT_COUNT];
#endif

#if (SPOT_LIGHT_COUNT > 0)
varying vec3 v_vertexToSpotLightDirection[SPOT_LIGHT_COUNT];
#if defined(BUMPED)
varying vec3 v_spotLightDirection[SPOT_LIGHT_COUNT];
#endif
#endif

#if defined(SPECULAR)
varying vec3 v_cameraDirection;
#endif

#include "lighting.frag"

#endif

#if defined(CLIP_PLANE)
varying float v_clipDistance;
#endif

#if defined(REFLECT)
varying float v_clipDistance;
#endif

#ifdef REFLECT

uniform vec4 u_reflectTintColor;
uniform float u_reflectTintFactor;

const vec3 Xunitvec = vec3(1.0, 0.0, 0.0);
const vec3 Yunitvec = vec3(0.0, 1.0, 0.0);

#endif


#ifdef CUBE_MAP
uniform samplerCube u_cubeTexture;
varying vec3 v_cubeEyePosition;
varying vec3 v_cubeNormalVector;
#endif

#ifdef CUBE_MAP_ALPHA
uniform float u_cubeTextureAlpha;
#endif



void main()
{
    
    _baseColor = vec4(0.0);
    
    
#ifdef REFLECT
    
    // basic reflection
    vec3 reflectDir = reflect(v_cameraDirection, v_normalVector);
    vec2 texcoord;
    texcoord.x = dot(normalize(reflectDir), Xunitvec) ;
    texcoord.y = dot(normalize(reflectDir), Yunitvec);
    
    texcoord.x = (-texcoord.x) * 0.5 + 1.0;
    texcoord.y = 1.0 - (texcoord.y + 1.0) * 0.5;
    
    
    vec4 texColor = texture2D(u_diffuseTexture, texcoord);
    _baseColor = mix(texColor, u_reflectTintColor, u_reflectTintFactor);

#else
    
#ifndef IGNORE_DIFFUSE_TEXTURE
    _baseColor = texture2D(u_diffuseTexture, v_texCoord);
#endif
    
#endif
    
    
#ifdef CUBE_MAP

#if defined(BUMPED)
    vec3 normalVector = normalize(texture2D(u_normalmapTexture, v_texCoord).rgb * 2.0 - 1.0);
    normalVector = normalize(normalVector * u_normalMixFactor + v_normalVector*(1.0 - u_normalMixFactor));
#else
    vec3 normalVector = normalize(v_normalVector);
#endif
    vec3 reflectDir = reflect(v_cubeEyePosition, normalize(normalVector));

    vec4 reflectDirAfterDeviceRoll = u_deviceInvRoll * vec4(reflectDir, 0.0);
    reflectDirAfterDeviceRoll.y = -reflectDirAfterDeviceRoll.y;
    reflectDir = normalize(reflectDirAfterDeviceRoll.xyz);

    vec4 color = textureCube(u_cubeTexture, vec3(reflectDir.x, reflectDir.y, reflectDir.z));
#ifdef CUBE_MAP_ALPHA
    color.a *= u_cubeTextureAlpha;
#endif
    _baseColor = vec4(_baseColor.rgb * (1.0 - color.a) + color.rgb * color.a, _baseColor.a);
#endif
    
    
    
#ifdef CHROMAKEY
    if(_baseColor.g > (_baseColor.r + _baseColor.b) * 3.0) {
        _baseColor = vec4(0.0);
    }
#endif
    
#ifndef PREMULTIPLY
    _baseColor = vec4(_baseColor.rgb * _baseColor.a, _baseColor.a);
#endif
    
    gl_FragColor = _baseColor;
    
    
#if defined(TEXTURE_DISCARD_ALPHA)
    if (gl_FragColor.a < 0.5)
        discard;
#endif
    
#if defined(LIGHTING) && !defined(REFLECT)
    
    gl_FragColor.rgb = getLitPixel();
#else
    gl_FragColor.rgb = _baseColor.rgb;
#endif
    
#if defined(LIGHTMAP)
    vec4 lightColor = texture2D(u_lightmapTexture, v_texCoord1);
    gl_FragColor.rgb *= lightColor.rgb;
#endif
    
#if defined(MODULATE_COLOR)
    gl_FragColor *= u_modulateColor;
#endif
    
#if defined(MODULATE_ALPHA)
    gl_FragColor.a *= u_modulateAlpha;
#endif
}
