package com.linecorp.line.camera.controller.function.record

import com.linecorp.line.media.analytics.MediaGACustomDimensionType
import com.linecorp.line.media.analytics.MediaGAEvents
import com.linecorp.yuki.camera.effect.android.model.FaceStickerModel
import com.linecorp.yuki.camera.effect.android.model.ScreenFilterModel
import com.linecorp.yuki.camera.effect.android.util.CameraConst
import jp.naver.line.androie.analytics.ga.GACustomDimensions
import jp.naver.line.androie.common.access.LineAccessForCommonHelper

class GaManager {
    private var screenFilterModel: ScreenFilterModel? = null
    private var screenFilterModelStr: String? = null
    private var faceStickerModel: FaceStickerModel? = null
    private lateinit var facing: CameraConst.Facing
    private lateinit var gaType: String

    private var distortionDensity = -1

    fun setScreenFilter(screenFilter: ScreenFilterModel?) {
        this.screenFilterModel = screenFilter
    }

    fun setScreenFilterModelStr(screenFilterModelStr: String?) {
        this.screenFilterModelStr = screenFilterModelStr
    }

    fun setFaceSticker(faceSticker: FaceStickerModel?) {
        this.faceStickerModel = faceSticker
    }

    fun setDistortionDensity(distortionDensity: Int) {
        this.distortionDensity = distortionDensity
    }

    fun setGaType(gaType: String) {
        this.gaType = gaType
    }

    fun setFacing(facing: CameraConst.Facing) {
        this.facing = facing
    }

    fun sendGaGifShootEvent() {
        val mediaGAEvents = if (facing === CameraConst.Facing.FRONT) {
            MediaGAEvents.MEDIA_CAMERA_GIF_SHOOT_FRONT
        } else {
            MediaGAEvents.MEDIA_CAMERA_GIF_SHOOT_BACK
        }
        sendGaEvent(mediaGAEvents)
    }

    fun sendGaGifLongShootEvent() {
        val mediaGAEvents = if (facing === CameraConst.Facing.FRONT) {
            MediaGAEvents.MEDIA_CAMERA_GIF_LONG_SHOOT_FRONT
        } else {
            MediaGAEvents.MEDIA_CAMERA_GIF_LONG_SHOOT_BACK
        }
        sendGaEvent(mediaGAEvents)
    }

    fun sendGaPhotoEvent() {
        val mediaGAEvents = if (facing === CameraConst.Facing.FRONT)
            MediaGAEvents.MEDIA_CAMERA_SHOOT_PHOTO_FRONT
        else
            MediaGAEvents.MEDIA_CAMERA_SHOOT_PHOTO_BACK
        sendGaEvent(mediaGAEvents)
    }

    fun sendGaVideoEvent() {
        val mediaGAEvents = if (facing === CameraConst.Facing.FRONT)
            MediaGAEvents.MEDIA_CAMERA_SHOOT_VIDEO_FRONT
        else
            MediaGAEvents.MEDIA_CAMERA_SHOOT_VIDEO_BACK
        sendGaEvent(mediaGAEvents)
    }

    fun sendGaEvent(mediaGAEvents: MediaGAEvents) {
        val gaCustomDimensions = GACustomDimensions()
        gaCustomDimensions.add(MediaGACustomDimensionType.MEDIA_LOCATION.dimensionNumber, gaType)

        screenFilterModel?.let {
            if (screenFilterModel?.yukiFilter != null) {
                gaCustomDimensions.add(MediaGACustomDimensionType.CAMERA_FILTER_NAME.dimensionNumber, screenFilterModelStr)
            }
        }

        faceStickerModel?.let {
            gaCustomDimensions.add(MediaGACustomDimensionType.CAMERA_EFFECT_NAME.dimensionNumber, faceStickerModel?.yukiSticker?.name)
            val intensity = if (faceStickerModel?.isAdjustableDistortion == true) "none" else distortionDensity.toString()

            gaCustomDimensions.add(MediaGACustomDimensionType.CAMERA_EFFECT_DISTORTION.dimensionNumber, intensity)
        }

        LineAccessForCommonHelper.getLineAccess().sendGaEvent(
                mediaGAEvents.category,
                mediaGAEvents.action,
                mediaGAEvents.label,
                gaCustomDimensions, null)
    }
}
