package com.linecorp.line.camera.controller.function.record

import android.app.Activity
import android.media.AudioManager
import android.media.SoundPool
import android.util.Log
import com.linecorp.line.media.BuildConfig
import com.linecorp.line.media.R

class RecordSoundManager {

    private var soundPool: SoundPool? = null

    private val soundEffectHashMap = HashMap<Int, Int>()

    companion object {
        const val TAG = "SoundManager"
        const val PLAY_SOUND = 0
        const val PLAY_STOP = 1
        const val PLAY_SHUTTER = 2
        const val PLAY_GIF_SHUTTER = 3

        const val PLAY_SOUND_DURATION_MILLISECONDS = 500L
    }

    fun initPlayer(activity: Activity) {
        //Some device(etc, Samsung devices) doesn't work what we expect.
        //Therefore, we only use the old api for SoundPool.
        /*
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            val audio = AudioAttributes.Builder()
                    .setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION)
                    .setFlags(AudioAttributes.FLAG_AUDIBILITY_ENFORCED)
                    .setUsage(AudioAttributes.CONTENT_TYPE_SONIFICATION)
                    .build()

            soundPool = SoundPool.Builder().setAudioAttributes(audio).build()
        } else {
            @Suppress("DEPRECATION")
            soundPool = SoundPool(1, AudioManager.STREAM_SYSTEM, 0)
        }
        */

        @Suppress("DEPRECATION")
        soundPool = SoundPool(1, AudioManager.STREAM_SYSTEM, 0)

        soundEffectHashMap.put(PLAY_SOUND, soundPool?.load(activity, R.raw.video_start_0613, 1) ?: 0)
        soundEffectHashMap.put(PLAY_STOP, soundPool?.load(activity, R.raw.video_end_0613, 1) ?: 0)
        soundEffectHashMap.put(PLAY_SHUTTER, soundPool?.load(activity, R.raw.camera_shutter_0613, 1) ?: 0)
        soundEffectHashMap.put(PLAY_GIF_SHUTTER, soundPool?.load(activity, R.raw.gif_camera_shutter, 1) ?: 0)
    }

    fun play(index: Int): Int {
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "play() index : " + index)
        }
        if (!isValidSound(index)) {
            return 0
        }
        return soundPool?.play(soundEffectHashMap[index] ?: 0, 1.0f, 1.0f, 1, 0, 1.0f) ?: 0
    }

    fun playLoopShutter(index: Int): Int {
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "playLoopShutter(): " + index)
        }
        if (!isValidSound(index)) {
            return 0
        }
        return soundPool?.play(soundEffectHashMap[index] ?: 0, 1.0f, 1.0f, 1, -1, 1.0f) ?: 0
    }

    fun stop(streamId: Int = 0) {
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "stop streamId: " + streamId)
        }
        if (!isValidSound(streamId)) {
            return
        }
        soundPool?.stop(streamId)
    }

    fun release() {
        soundPool?.release()
        soundPool = null
    }

    private fun isValidSound(index: Int): Boolean {
        val soundId = soundEffectHashMap[index]
        val isValid = soundId != 0
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "isValidSound(): index : $index, soundId : $soundId, isValid : $isValid")
        }
        return isValid
    }
}
