package com.linecorp.line.camera.controller.function.record.action

import android.app.Activity
import com.linecorp.line.camera.controller.function.record.GaManager
import com.linecorp.line.camera.controller.function.record.RecordSoundManager
import com.linecorp.line.camera.core.ILineCamera
import com.linecorp.line.camera.model.CameraConstExt
import com.linecorp.line.media.R

class GifRecordAction : RecordAction {
    private lateinit var lineCamera: ILineCamera
    private var recordGifState: CameraConstExt.RecordGifState = CameraConstExt.RecordGifState.READY
    private lateinit var recordSoundManager: RecordSoundManager
    private lateinit var gaManager: GaManager

    private var ignoreKeyEvent = true
    private var streamId = 0

    override fun init(activity: Activity?) = Unit

    override fun stopRecording() {
        lineCamera.stopRecordGif()
    }

    override fun cancelRecording() {
        recordSoundManager.stop(streamId)
        lineCamera.cancelRecordGif()
    }

    override fun takeShortRecord() {
        takeGifShortRecord()
    }

    override fun takeLongRecord() {
        if (ignoreKeyEvent) {
            return
        }
        setIgnoreKeyEvent(true)
        lineCamera.startRecordGif(false)

        gaManager.sendGaGifLongShootEvent()
    }

    override fun onChangedGifCameraStatus(recordGifState: CameraConstExt.RecordGifState) {
        this.recordGifState = recordGifState

        when (recordGifState) {
            CameraConstExt.RecordGifState.RECORDING -> {
                streamId = recordSoundManager.playLoopShutter(RecordSoundManager.PLAY_GIF_SHUTTER)
            }
            CameraConstExt.RecordGifState.STOP, CameraConstExt.RecordGifState.READY -> {
                recordSoundManager.stop(streamId)
                setIgnoreKeyEvent(false)
            }
            else -> {
            }
        }
    }

    override fun onKeyDown(): Boolean {
        return takeGifShortRecord()
    }

    private fun takeGifShortRecord(): Boolean {
        if (ignoreKeyEvent) {
            return false
        }
        lineCamera.startRecordGif(true)
        setIgnoreKeyEvent(true)

        gaManager.sendGaGifShootEvent()
        return true
    }

    override fun cancelCountdown(): Boolean {
        return if (recordGifState === CameraConstExt.RecordGifState.START_LONG_SHOT) {
            false
        } else {
            if (lineCamera.requestCancelTimer()) {
                setIgnoreKeyEvent(false)
                return true
            }
            return false
        }
    }

    override fun setLineCamera(lineCamera: ILineCamera) {
        this.lineCamera = lineCamera
    }

    override fun setIgnoreKeyEvent(ignoreKeyEvent: Boolean) {
        this.ignoreKeyEvent = ignoreKeyEvent
    }

    override fun setSoundManager(recordSoundManager: RecordSoundManager) {
        this.recordSoundManager = recordSoundManager
    }

    override fun setGaManager(gaManager: GaManager) {
        this.gaManager = gaManager
    }

    override fun getNeedLongPressed(): Boolean {
        return lineCamera.isSupportGifCamera
    }

    override fun isNeedToCancelRecording(): Boolean {
        return lineCamera.recordGifStatus === CameraConstExt.RecordGifState.RECORDING
    }

    override fun getA11yStrRes(): Int {
        // todo
        return R.string.access_camera_shoot // todo
    }
}