package com.linecorp.line.camera.core.gif.timer

import android.os.Handler
import android.os.Message
import android.util.Log
import com.linecorp.line.media.BuildConfig

class GifCameraTimer : Handler() {
    companion object {
        const val TAG = "GifCameraTimer"
    }

    private lateinit var onGifCameraTimerCountDownListener: OnGifCameraTimerCountDownListener
    private var isCanceled = false
    private var time = 0

    fun registerCountDownListener(onGifCameraTimerCountDownListener: OnGifCameraTimerCountDownListener) {
        this.onGifCameraTimerCountDownListener = onGifCameraTimerCountDownListener
    }

    fun startTimer(time: Long) {
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "startTimer: ")
        }
        this.isCanceled = false
        this.time = time.toInt()
        if (time <= 0) {
            onGifCameraTimerCountDownListener.onCompleteCountDown()
        } else {
            val message = Message.obtain()
            message.what = GifCameraTimerMessage.MSG_COUNTDOWN
            sendMessage(message)
        }
    }

    fun requestCancel() {
        isCanceled = true
        onGifCameraTimerCountDownListener.onCountDownCanceled()
        removeMessages(GifCameraTimerMessage.MSG_COUNTDOWN)
    }

    fun isRunning(): Boolean = hasMessages(GifCameraTimerMessage.MSG_COUNTDOWN)

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)

        if (isCanceled) {
            if (BuildConfig.DEBUG) {
                Log.d(TAG, "handleMessage canceled: " + isCanceled)
            }
            return
        }

        when (msg?.what) {
            GifCameraTimerMessage.MSG_COUNTDOWN -> {
                if (time == 0) {
                    if (BuildConfig.DEBUG) {
                        Log.d(TAG, "handleMessage: " + time)
                    }
                    onGifCameraTimerCountDownListener.onCompleteCountDown()
                } else {
                    if (BuildConfig.DEBUG) {
                        Log.d(TAG, "handleMessage: " + time)
                    }
                    onGifCameraTimerCountDownListener.onCountDown(time--)

                    val message = Message.obtain()
                    message.what = GifCameraTimerMessage.MSG_COUNTDOWN

                    sendMessageDelayed(message, GifCameraTimerMessage.DELAY)
                }
            }
        }
    }
}