@file: JvmName("SurfaceViewSizeCalculator")

package com.linecorp.line.camera.model

import android.app.Activity
import android.content.Context
import com.linecorp.yuki.camera.effect.android.util.CameraConst
import jp.naver.line.androie.common.util.toPixel

fun getSurfaceViewSize(activity: Activity, surfaceViewSize: SurfaceViewSize, ratio: CameraConst.Ratio) {
    val display = activity.windowManager.defaultDisplay
    @Suppress("DEPRECATION")
    val displayWidth: Int = display.width
    @Suppress("DEPRECATION")
    val displayHeight: Int = display.height

    surfaceViewSize.width = displayWidth
    surfaceViewSize.height = displayHeight
    surfaceViewSize.topMargin = getTopMargin(activity, ratio)

    // Camera is always portrait. But camera can have the landscape orientation when opened from "landscape orientation -> portrait caller -> landscape picker".
    // This is a workaround for late changed orientation.
    if (displayWidth > displayHeight) {
        when (ratio) {
            CameraConst.Ratio.RATIO_16x9 -> {
                surfaceViewSize.width = displayHeight * 16 / 9
            }
            CameraConst.Ratio.RATIO_4x3 -> {
                surfaceViewSize.leftMargin = (displayWidth - displayHeight * 4 / 3) / 2
                surfaceViewSize.rightMargin = (displayWidth - displayHeight * 4 / 3) / 2
            }
            CameraConst.Ratio.RATIO_1x1 -> {
                surfaceViewSize.leftMargin = (displayWidth - displayHeight) / 2
                surfaceViewSize.rightMargin = (displayWidth - displayHeight) / 2
            }
            else -> return
        }
        return
    }

    surfaceViewSize.height = when (ratio) {
        CameraConst.Ratio.RATIO_16x9 -> {
            displayWidth * 16 / 9
        }
        CameraConst.Ratio.RATIO_4x3 -> {
            displayWidth * 4 / 3
        }
        CameraConst.Ratio.RATIO_1x1 -> {
            displayWidth
        }
        else -> 0
    }
}

fun getTopMargin(context: Context, ratio: CameraConst.Ratio): Int {
    return when (ratio) {
        CameraConst.Ratio.RATIO_1x1 -> toPixel(context, 50f)
        else -> 0
    }
}

data class SurfaceViewSize(var width: Int = 0,
                           var height: Int = 0,
                           var leftMargin: Int = 0,
                           var rightMargin: Int = 0,
                           var topMargin: Int = 0,
                           var bottomMargin: Int = 0)
