package com.linecorp.line.camera.view.record

import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.util.Log
import com.linecorp.line.media.BuildConfig
import jp.naver.line.androie.common.util.toPixel

class DrawRecordingViewUtils {
    private val progressPaint = Paint().apply {
        style = Paint.Style.STROKE
        strokeWidth = CIRCLE_WIDTH
        isAntiAlias = true
        color = Color.parseColor("#ffffff")
        strokeCap = Paint.Cap.ROUND
    }

    private val progressCircleBgPaint = Paint().apply {
        style = Paint.Style.STROKE
        strokeWidth = CIRCLE_WIDTH
        isAntiAlias = true
        color = Color.parseColor("#10ffffff")
        strokeCap = Paint.Cap.ROUND
    }

    private val circlePaint = Paint().apply {
        style = Paint.Style.STROKE
        strokeWidth = CIRCLE_WIDTH
        isAntiAlias = true
        color = Color.WHITE
        setShadowLayer(20f, 0f, 0f, Color.BLACK)
        strokeCap = Paint.Cap.ROUND
    }

    private val circleStrokePaint = Paint().apply {
        style = Paint.Style.STROKE
        strokeWidth = CIRCLE_STOKE_WIDTH
        isAntiAlias = true
        color = Color.GRAY
        strokeCap = Paint.Cap.ROUND
    }

    private val rectPaint = Paint().apply {
        style = Paint.Style.FILL
        isAntiAlias = true
        color = Color.WHITE
    }

    private val rectStrokePaint = Paint().apply {
        style = Paint.Style.FILL
        isAntiAlias = true
        color = Color.GRAY
    }

    private val circleWhite = HashMap<Int, Int>()
    private val circleBlack = HashMap<Int, Int>()
    private val circleWithRectWhite = HashMap<Int, Int>()
    private val circleWithRectWhitePressed = HashMap<Int, Int>()
    private val circleWithRectBlack = HashMap<Int, Int>()
    private val circleWithRectBlackPressed = HashMap<Int, Int>()
    private val progressWhite = HashMap<Int, Int>()
    private val progressBlack = HashMap<Int, Int>()

    fun createProgressPaint(): Paint = progressPaint

    fun createProgressCircleBgPaint(): Paint = progressCircleBgPaint

    fun createCirclePaint(): Paint = circlePaint

    fun createCircleStrokePaint(): Paint = circleStrokePaint

    fun createRectPaint(): Paint = rectPaint

    fun createRectStrokePaint(): Paint = rectStrokePaint

    fun drawRect(canvas: Canvas?, w: Int, h: Int, rectPaint: Paint?, rectStrokePaint: Paint?) {
        val left = w / 2 - DrawRecordingViewUtils.RECT_WIDTH / 2
        val top = h / 2 - DrawRecordingViewUtils.RECT_WIDTH / 2
        val right = w / 2 + DrawRecordingViewUtils.RECT_WIDTH
        val bottom = h / 2 + DrawRecordingViewUtils.RECT_WIDTH

        canvas?.drawRect(left - DrawRecordingViewUtils.RECT_STROKE_WIDTH, top - DrawRecordingViewUtils.RECT_STROKE_WIDTH, right + DrawRecordingViewUtils.RECT_STROKE_WIDTH, bottom + DrawRecordingViewUtils.RECT_STROKE_WIDTH, rectStrokePaint)
        canvas?.drawRect(left, top, right, bottom, rectPaint)
    }

    fun drawDashArcsNoAnim(canvas: Canvas?, rectF: RectF, paint: Paint) {
        var x = 0f
        var startAngle = 268f - DASH_SHUTTER_ARC_WIDTH
        val arcCount = Math.round(360 / LINE_DASH_CIRCLE_WIDTH)

        while (x <= arcCount) {
            canvas?.drawArc(rectF, startAngle, DASH_SHUTTER_ARC_WIDTH, false, paint)
            startAngle += LINE_DASH_CIRCLE_WIDTH
            startAngle %= 360
            x++
        }
    }

    fun drawArcs(canvas: Canvas?, rectF: RectF, circleArcWidth: Float, width1: Float, width2: Float, sweepAngleOffset: Float, paint: Paint) {
        var x = 0f
        var startAngle = 268f
        val arcCount = Math.round(360 / circleArcWidth)

        if (BuildConfig.DEBUG) {
            Log.d(TAG, "drawArcs: " + arcCount)
        }

        while (x <= arcCount) {
            val startAngleOffset = startAngle - (width1 - sweepAngleOffset)
            if (BuildConfig.DEBUG) {
                Log.d(TAG, "drawArcs: startAngleOffset: $startAngleOffset/$startAngle")
            }
            canvas?.drawArc(rectF, startAngleOffset, sweepAngleOffset, false, paint)
            startAngle += width2
            startAngle %= 360f
            x++
        }
    }

    fun drawDashArcs(canvas: Canvas?, rectF: RectF, circleArcWidth: Float, width: Float, sweepAngleOffset: Float, paint: Paint) {
        var x = 0f
        var startAngle = 268f
        val arcCount = Math.round(360 / circleArcWidth)

        if (BuildConfig.DEBUG) {
            Log.d(TAG, "drawArcs: " + arcCount)
        }

        while (x <= arcCount) {
            val startAngleOffset = startAngle + (width - sweepAngleOffset)
            if (BuildConfig.DEBUG) {
                Log.d(TAG, "drawArcs: startAngleOffset: $startAngleOffset/$startAngle")
            }
            canvas?.drawArc(rectF, startAngleOffset, sweepAngleOffset, false, paint)
            startAngle += width
            startAngle %= 360f
            x++
        }
    }

    fun getShootCircleColor(viewColor: RecordingViewMode.ViewColor): Int =
            getValue(viewColor, circleWhite, circleBlack, SHOOT_CIRCLE)

    fun getShootCircleShadowColor(viewColor: RecordingViewMode.ViewColor): Int =
            getValue(viewColor, circleWhite, circleBlack, SHOOT_CIRCLE_SHADOW)

    fun getShootCircleWithRectColor(viewMode: RecordingViewMode.ViewMode, viewColor: RecordingViewMode.ViewColor): Int =
            when (viewMode) {
                RecordingViewMode.ViewMode.RECORDING_VIDEO_PRESSED -> {
                    getValue(viewColor, circleWithRectWhitePressed, circleWithRectBlackPressed, SHOOT_CIRCLE)
                }
                RecordingViewMode.ViewMode.RECORDING_VIDEO_NON_PRESSED -> {
                    getValue(viewColor, circleWithRectWhite, circleWithRectBlack, SHOOT_CIRCLE)
                }
                else -> {
                    1
                }
            }

    fun getShootCircleShadowWithRectColor(viewMode: RecordingViewMode.ViewMode, viewColor: RecordingViewMode.ViewColor): Int =
            when (viewMode) {
                RecordingViewMode.ViewMode.RECORDING_VIDEO_PRESSED -> {
                    getValue(viewColor, circleWithRectWhitePressed, circleWithRectBlackPressed, SHOOT_CIRCLE_SHADOW)
                }
                RecordingViewMode.ViewMode.RECORDING_VIDEO_NON_PRESSED -> {
                    getValue(viewColor, circleWithRectWhite, circleWithRectBlack, SHOOT_CIRCLE_SHADOW)
                }
                else -> {
                    1
                }
            }

    fun getShootRectInsideCircle(viewMode: RecordingViewMode.ViewMode, viewColor: RecordingViewMode.ViewColor): Int =
            when (viewMode) {
                RecordingViewMode.ViewMode.RECORDING_VIDEO_PRESSED -> {
                    getValue(viewColor, circleWithRectWhitePressed, circleWithRectBlackPressed, SHOOT_RECT_INSIDE_CIRCLE)
                }
                RecordingViewMode.ViewMode.RECORDING_VIDEO_NON_PRESSED, RecordingViewMode.ViewMode.STOP_SHOT_GIF -> {
                    getValue(viewColor, circleWithRectWhite, circleWithRectBlack, SHOOT_RECT_INSIDE_CIRCLE)
                }
                else -> {
                    1
                }
            }

    fun getShootRectInsideCircleShadow(viewMode: RecordingViewMode.ViewMode, viewColor: RecordingViewMode.ViewColor): Int =
            when (viewMode) {
                RecordingViewMode.ViewMode.RECORDING_VIDEO_PRESSED -> {
                    getValue(viewColor, circleWithRectWhitePressed, circleWithRectBlackPressed, SHOOT_RECT_INSIDE_CIRCLE_SHADOW)
                }
                RecordingViewMode.ViewMode.RECORDING_VIDEO_NON_PRESSED -> {
                    getValue(viewColor, circleWithRectWhite, circleWithRectBlack, SHOOT_RECT_INSIDE_CIRCLE_SHADOW)
                }
                else -> {
                    1
                }
            }

    fun getCircleInteractColor(viewMode: RecordingViewMode.ViewMode, viewColor: RecordingViewMode.ViewColor): Int =
            when (viewMode) {
                RecordingViewMode.ViewMode.RECORDING_VIDEO_PRESSED -> {
                    getValue(viewColor, circleWithRectWhitePressed, circleWithRectBlackPressed, SHOOT_CIRCLE_INTERACT)
                }
                RecordingViewMode.ViewMode.RECORDING_VIDEO_NON_PRESSED -> {
                    getValue(viewColor, circleWithRectWhite, circleWithRectBlack, SHOOT_CIRCLE_INTERACT)
                }
                else -> {
                    1
                }
            }

    fun getProgressCircleColor(viewColor: RecordingViewMode.ViewColor): Int =
            getValue(viewColor, progressWhite, progressBlack, SHOOT_CIRCLE_PROGRESS)

    fun getProgressCircleBgColor(viewColor: RecordingViewMode.ViewColor): Int =
            getValue(viewColor, progressWhite, progressBlack, SHOOT_CIRCLE_PROGRESS_BG)

    private fun getValue(viewColor: RecordingViewMode.ViewColor, whiteMap: HashMap<Int, Int>, blackMap: HashMap<Int, Int>, index: Int): Int =
            when (viewColor) {
                RecordingViewMode.ViewColor.WHITE -> {
                    whiteMap[index] as Int
                }
                RecordingViewMode.ViewColor.BLACK -> {
                    blackMap[index] as Int
                }
            }

    companion object {
        private val TAG = "DrawRecordingViewUtils"

        private val SHOOT_CIRCLE = 1
        private val SHOOT_CIRCLE_SHADOW = 2
        private val SHOOT_RECT_INSIDE_CIRCLE = 3
        private val SHOOT_RECT_INSIDE_CIRCLE_SHADOW = 4
        private val SHOOT_CIRCLE_INTERACT = 5
        private val SHOOT_CIRCLE_PROGRESS = 6
        private val SHOOT_CIRCLE_PROGRESS_BG = 7

        const val OUTER_CIRCLE_RATIO = 2.2f
        const val ANIMATION_DURATION = 2000L
        const val PROGRESS_ANIMATION_DURATION = 5

        @Suppress("DEPRECATION")
        val RECT_WIDTH = toPixel(10f).toFloat()
        @Suppress("DEPRECATION")
        val RECT_STROKE_WIDTH = toPixel(0.5f).toFloat()
        @Suppress("DEPRECATION")
        val CIRCLE_WIDTH = toPixel(6f).toFloat()
        @Suppress("DEPRECATION")
        val CIRCLE_STOKE_WIDTH = toPixel(7f).toFloat()
        val CIRCLE_WIDTH_PADDING = CIRCLE_WIDTH
        const val DASH_SHUTTER_ARC_PADDING_RIGHT = 20f // todo design
        const val DASH_SHUTTER_ARC_WIDTH = 10f // todo design
        const val LINE_DASH_CIRCLE_WIDTH = DASH_SHUTTER_ARC_PADDING_RIGHT + DASH_SHUTTER_ARC_WIDTH // todo design
    }

    init {
        circleWhite.put(SHOOT_CIRCLE, Color.WHITE)
        circleWhite.put(SHOOT_CIRCLE_SHADOW, Color.GRAY)
        circleWhite.put(SHOOT_RECT_INSIDE_CIRCLE, 1)
        circleWhite.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, 1)
        circleWhite.put(SHOOT_CIRCLE_INTERACT, 1)
        circleWhite.put(SHOOT_CIRCLE_PROGRESS, 1)
        circleWhite.put(SHOOT_CIRCLE_PROGRESS_BG, 1)
        circleBlack.put(SHOOT_CIRCLE, Color.BLACK)
        circleBlack.put(SHOOT_CIRCLE_SHADOW, Color.BLACK)
        circleBlack.put(SHOOT_RECT_INSIDE_CIRCLE, 1)
        circleBlack.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, 1)
        circleBlack.put(SHOOT_CIRCLE_INTERACT, 1)
        circleBlack.put(SHOOT_CIRCLE_PROGRESS, 1)
        circleBlack.put(SHOOT_CIRCLE_PROGRESS_BG, 1)
        circleWithRectWhite.put(SHOOT_CIRCLE, Color.WHITE)
        circleWithRectWhite.put(SHOOT_CIRCLE_SHADOW, Color.GRAY)
        circleWithRectWhite.put(SHOOT_RECT_INSIDE_CIRCLE, Color.WHITE)
        circleWithRectWhite.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, Color.GRAY)
        circleWithRectWhite.put(SHOOT_CIRCLE_INTERACT, Color.parseColor("#66000000"))
        circleWithRectWhite.put(SHOOT_CIRCLE_PROGRESS, 1)
        circleWithRectWhite.put(SHOOT_CIRCLE_PROGRESS_BG, 1)
        circleWithRectBlack.put(SHOOT_CIRCLE, Color.DKGRAY)
        circleWithRectBlack.put(SHOOT_CIRCLE_SHADOW, Color.DKGRAY)
        circleWithRectBlack.put(SHOOT_RECT_INSIDE_CIRCLE, Color.BLACK)
        circleWithRectBlack.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, Color.BLACK)
        circleWithRectBlack.put(SHOOT_CIRCLE_INTERACT, Color.parseColor("#fff7f7f7"))
        circleWithRectBlack.put(SHOOT_CIRCLE_PROGRESS, 1)
        circleWithRectBlack.put(SHOOT_CIRCLE_PROGRESS_BG, 1)
        circleWithRectWhitePressed.put(SHOOT_CIRCLE, Color.parseColor("#50ffffff"))
        circleWithRectWhitePressed.put(SHOOT_CIRCLE_SHADOW, Color.parseColor("#50000000"))
        circleWithRectWhitePressed.put(SHOOT_RECT_INSIDE_CIRCLE, Color.parseColor("#50ffffff"))
        circleWithRectWhitePressed.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, Color.parseColor("#50000000"))
        circleWithRectWhitePressed.put(SHOOT_CIRCLE_INTERACT, Color.parseColor("#66000000"))
        circleWithRectWhitePressed.put(SHOOT_CIRCLE_PROGRESS, 1)
        circleWithRectWhitePressed.put(SHOOT_CIRCLE_PROGRESS_BG, 1)
        circleWithRectBlackPressed.put(SHOOT_CIRCLE, Color.LTGRAY)
        circleWithRectBlackPressed.put(SHOOT_CIRCLE_SHADOW, Color.LTGRAY)
        circleWithRectBlackPressed.put(SHOOT_RECT_INSIDE_CIRCLE, Color.DKGRAY)
        circleWithRectBlackPressed.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, Color.DKGRAY)
        circleWithRectBlackPressed.put(SHOOT_CIRCLE_INTERACT, Color.parseColor("#fff7f7f7"))
        circleWithRectBlackPressed.put(SHOOT_CIRCLE_PROGRESS, 1)
        circleWithRectBlackPressed.put(SHOOT_CIRCLE_PROGRESS_BG, 1)
        progressWhite.put(SHOOT_CIRCLE_PROGRESS, Color.parseColor("#ffffff"))
        progressWhite.put(SHOOT_CIRCLE_PROGRESS_BG, Color.parseColor("#10ffffff"))
        progressWhite.put(SHOOT_CIRCLE, 1)
        progressWhite.put(SHOOT_CIRCLE_SHADOW, 1)
        progressWhite.put(SHOOT_RECT_INSIDE_CIRCLE, 1)
        progressWhite.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, 1)
        progressWhite.put(SHOOT_CIRCLE_INTERACT, 1)
        progressBlack.put(SHOOT_CIRCLE_PROGRESS, Color.parseColor("#000000"))
        progressBlack.put(SHOOT_CIRCLE_PROGRESS_BG, Color.parseColor("#10000000"))
        progressBlack.put(SHOOT_CIRCLE, 1)
        progressBlack.put(SHOOT_CIRCLE_SHADOW, 1)
        progressBlack.put(SHOOT_RECT_INSIDE_CIRCLE, 1)
        progressBlack.put(SHOOT_RECT_INSIDE_CIRCLE_SHADOW, 1)
        progressBlack.put(SHOOT_CIRCLE_INTERACT, 1)
    }
}
