package com.linecorp.line.camera.view.record

import android.content.Context
import android.graphics.Canvas
import android.support.v7.widget.AppCompatImageButton
import android.util.AttributeSet
import android.util.Log
import com.linecorp.line.media.BuildConfig

class RecordingAnimationView @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : AppCompatImageButton(context, attrs, defStyleAttr), RecordingViewOperator {
    companion object {
        private const val TAG = "RecordingAnimationView"
    }

    private var recordingAnimationViewController: RecordingAnimationViewController = RecordingAnimationViewController(this)
    private var w: Float = 0.toFloat()
    private var h: Float = 0.toFloat()

    override fun startAnimation(viewMode: RecordingViewMode.ViewMode) {
        recordingAnimationViewController.startAnimation(viewMode)
    }

    override fun clearRecordingAnimation() {
        recordingAnimationViewController.clearRecordingAnimation()
    }

    override fun updateColor(viewColor: RecordingViewMode.ViewColor) {
        recordingAnimationViewController.updateColor(viewColor)
    }

    override fun updateProgressForGif(progress: Int, total: Int) {
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)

        if (w == 0 || h == 0) {
            return
        }

        if (BuildConfig.DEBUG) {
            Log.d("RecordingAnimationView", "onSizeChanged: $w/$h")
        }

        this.w = w.toFloat()
        this.h = h.toFloat()
        recordingAnimationViewController.onSizeUpdate(w, h)
    }

    override fun onDraw(canvas: Canvas) {
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "onDraw: ")
        }

        if (w == 0f || h == 0f) {
            if (BuildConfig.DEBUG) {
                Log.d(TAG, "onDraw: 0")
            }
            return
        }

        recordingAnimationViewController.onDraw(canvas, w, h)
    }
}
