package com.linecorp.line.media.picker.fragment.gif

import com.linecorp.line.media.R
import com.naver.mei.sdk.core.image.compositor.type.FrameAlignment
import com.naver.mei.sdk.core.image.meta.PlayDirection

class GIFMakerOptions {
    enum class Speed(val speed: Double, val description: String, val bigIcon: Int) {
        X1(1.0, "1X", R.drawable.gallery_ic_gif_speed_01),
        X2(2.0, "2X", R.drawable.gallery_ic_gif_speed_02),
        X3(3.0, "3X", R.drawable.gallery_ic_gif_speed_03),
        XM3(0.3, "-3X", R.drawable.gallery_ic_gif_slow_03),
        XM2(0.5, "-2X", R.drawable.gallery_ic_gif_slow_02);

        fun next(): Speed {
            val index: Int = values().indexOf(this)

            return when (index >= (values().size - 1)) {
                true -> values()[0]
                false -> values()[index + 1]
            }
        }
    }

    enum class Ratio(val alignment: FrameAlignment, val ratio: Double, val textRes: Int, val iconRes: Int, val bigIcon: Int) {
        ORIGINAL(FrameAlignment.KEEP_ORIGINAL_RATIO,
                3.0 / 4.0,
                R.string.gallery_creategif_resolution_original,
                R.drawable.selector_media_gif_maker_origin_button,
                R.drawable.gallery_ic_gif_origin),
        SQUARE(FrameAlignment.FIT_SHORT_AXIS_CENTER_CROP,
                1.0,
                R.string.gallery_creategif_resolution_square,
                R.drawable.selector_media_gif_maker_square_button,
                R.drawable.gallery_ic_gif_ratio);

        fun next(): Ratio = when (alignment) {
            FrameAlignment.KEEP_ORIGINAL_RATIO -> SQUARE
            FrameAlignment.FIT_SHORT_AXIS_CENTER_CROP -> ORIGINAL
        }
    }

    enum class Direction(val direction: PlayDirection, val textRes: Int, val iconRes: Int, val bigIcon: Int) {
        FORWARD(PlayDirection.FORWARD,
                R.string.gallery_creategif_label_forward,
                R.drawable.selector_media_gif_maker_forward_button,
                R.drawable.gallery_ic_gif_forward),
        REVERSE(PlayDirection.REVERSE,
                R.string.gallery_creategif_label_rewind,
                R.drawable.selector_media_gif_maker_rewind_button,
                R.drawable.gallery_ic_gif_rewind);

        fun next(): Direction = when (direction) {
            PlayDirection.FORWARD -> REVERSE
            PlayDirection.REVERSE -> FORWARD
            else -> FORWARD
        }
    }
}
