package com.linecorp.line.media.picker.fragment.gif

import android.content.DialogInterface
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.animation.AlphaAnimation
import android.view.animation.Animation
import android.webkit.MimeTypeMap
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.linecorp.line.camera.util.ContentsUtils
import com.linecorp.line.common.LineAccessForPickerHelper
import com.linecorp.line.common.LineMediaFileManager
import com.linecorp.line.common.PickerMediaItem
import com.linecorp.line.media.BuildConfig
import com.linecorp.line.media.R
import com.linecorp.line.media.analytics.MediaGACustomDimensionType
import com.linecorp.line.media.analytics.MediaGAEvents
import com.linecorp.line.media.picker.MediaPickerHelper
import com.linecorp.line.media.picker.MediaPickerInfo
import com.linecorp.line.media.picker.fragment.main.MediaPickerBaseFragment
import com.linecorp.line.media.picker.mode.gif.MediaGifMakerHelper.Companion.DEFAULT_DELAY
import com.linecorp.line.media.picker.subjects.param.MediaFragmentParam
import com.naver.mei.sdk.MeiSDK
import com.naver.mei.sdk.core.image.meta.PlayDirection
import com.naver.mei.sdk.error.MeiSDKErrorType
import com.naver.mei.sdk.listener.MeiEventListener
import com.naver.mei.sdk.view.MeiCanvasView
import jp.naver.gallery.android.media.MediaItem
import jp.naver.line.androie.analytics.ga.GACustomDimensions
import jp.naver.line.androie.common.access.LineAccessForCommonHelper
import jp.naver.line.androie.common.dialog.LineDialog
import jp.naver.line.androie.common.dialog.LineDialogHelper
import jp.naver.line.androie.common.dialog.LineProgressDialog
import jp.naver.line.androie.common.helper.DialogHelper
import jp.naver.line.androie.common.helper.ToastHelper
import jp.naver.line.androie.common.util.io.StorageUtils
import java.io.File
import java.text.SimpleDateFormat
import java.util.Date

class GIFMakersFragment : MediaPickerBaseFragment() {

    private lateinit var rootView: View
    private lateinit var bottomButtonView: ViewGroup
    private lateinit var bigIcon: ImageView
    private lateinit var sendButton: TextView
    private lateinit var imagesPaths: List<String>
    private lateinit var mode: MediaPickerInfo.MODE
    private var isFromCamera: Boolean = false
    private var cameraRatio: Double = 1.0
    private var frameDelayMsec: Int = 500
    private var progressDialog: LineProgressDialog? = null
    private var errorDialog: LineDialog? = null

    var speed: GIFMakerOptions.Speed = GIFMakerOptions.Speed.X1
    var ratio: GIFMakerOptions.Ratio = GIFMakerOptions.Ratio.ORIGINAL
    var direction: GIFMakerOptions.Direction = GIFMakerOptions.Direction.FORWARD

    private val gifPreview: MeiCanvasView by lazy {
        rootView.findViewById<MeiCanvasView>(R.id.gif_preview)
    }

    private val speedButton: GIFMakerBottomButtonView by lazy {
        GIFMakerBottomButtonView(context).apply {
            setIcon(R.drawable.selector_media_gif_maker_speed_button)
            setText(speed.description)
            setOnClickListener { changeSpeed() }
            layoutParams = LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.MATCH_PARENT).apply {
                weight = 1.0f
            }
        }
    }

    private val ratioButton: GIFMakerBottomButtonView by lazy {
        GIFMakerBottomButtonView(context).apply {
            setIcon(ratio.iconRes)
            setText(ratio.textRes)
            setOnClickListener { changeAspectRatio() }
            layoutParams = LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.MATCH_PARENT).apply {
                weight = 1.0f
            }
        }
    }

    private val saveButton: GIFMakerBottomButtonView by lazy {
        GIFMakerBottomButtonView(context).apply {
            setIcon(R.drawable.selector_media_gif_maker_save_button)
            setText(R.string.gallery_save)
            setOnClickListener {
                activity?.let {
                    DialogHelper.showOptionsDialog(it,
                            it.getString(R.string.gallery_save),
                            saveItemTitle, saveDialogListener)
                }
            }
            layoutParams = LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.MATCH_PARENT).apply {
                weight = 1.0f
            }
        }
    }

    private val directionButton: GIFMakerBottomButtonView by lazy {
        GIFMakerBottomButtonView(context).apply {
            setIcon(direction.iconRes)
            setText(direction.textRes)
            setOnClickListener { changeDirection() }
            layoutParams = LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.MATCH_PARENT).apply {
                weight = 1.0f
            }
        }
    }

    private val saveItemTitle: List<String> by lazy {
        ArrayList<String>().apply {
            add(activity?.getString(R.string.gallery_save).orEmpty())
            add(activity?.getString(R.string.gallery_keepconnect_save).orEmpty())
        }
    }

    private val saveDialogListener: List<DialogInterface.OnClickListener> by lazy {
        ArrayList<DialogInterface.OnClickListener>().apply {
            add(DialogInterface.OnClickListener { _, _ -> save({ _ -> save() }, getSaveFileName()) })
            add(DialogInterface.OnClickListener { _, _ -> save({ path -> sendToKeep(path) }, getKeepFileName()) })
        }
    }

    class SaveEventListener(private val saveFun: (m: String) -> Unit,
                            val progressDialog: LineProgressDialog?,
                            private val errorDialog: LineDialog?) : MeiEventListener {
        override fun onSuccess(path: String?) {
            if (isSaveCancel) return
            progressDialog?.dismiss()
            path?.let { saveFun(it) }
        }

        override fun onFail(p0: MeiSDKErrorType?) {
            if (BuildConfig.DEBUG) {
                Log.e(TAG, "SaveEventListener.onFail():  ${p0?.code}, ${p0?.message}")
            }
            if (isSaveCancel) return
            progressDialog?.dismiss()
            errorDialog?.show()
        }

        override fun onProgress(progress: Double) {
            if (isSaveCancel) return
            progressDialog?.progress = (progress * 100).toInt()
        }
    }

    companion object {
        const val TAG = "GIFMakersFragment"
        const val MIN_AVAILABLE_STORAGE_BYTE = 128 * 1024 * 1024L // 128MB, The policy from meiSDK

        const val ARG_MEDIA_ITEMS = "mediaItems"
        const val ARG_PICKER_MODE = "pickerMode"
        const val ARG_FRAME_DELAY_MSEC = "frameDelayMsec"
        const val ARG_IS_FROM_CAMERA = "isFromCamera"
        const val ARG_CAMERA_RATIO = "cameraRatio"
        var isSaveCancel = false

        @JvmStatic
        fun newInstance(mode: MediaPickerInfo.MODE,
                        items: List<PickerMediaItem>,
                        frameDelayMsec: Int = DEFAULT_DELAY,
                        isFromCamera: Boolean = true,
                        cameraRatio: Double = 1.0): GIFMakersFragment {
            return GIFMakersFragment().also { fragment ->
                fragment.arguments = Bundle().apply {
                    putString(ARG_PICKER_MODE, mode.name)
                    putStringArrayList(ARG_MEDIA_ITEMS, ArrayList(items.map { it.resultImageUri.toString() }))
                    putInt(ARG_FRAME_DELAY_MSEC, frameDelayMsec)
                    putBoolean(ARG_IS_FROM_CAMERA, isFromCamera)
                    putDouble(ARG_CAMERA_RATIO, cameraRatio)
                }
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        initFields()
        MeiSDK.setMinimumStorageSpace(MIN_AVAILABLE_STORAGE_BYTE)
        MeiSDK.setStorageDir(ContentsUtils.getFolderOfLineDefaultCamera())
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        rootView = inflater.inflate(R.layout.fragment_media_gif_maker, container, false)
        bottomButtonView = rootView.findViewById(R.id.bottom_button_layout)
        bigIcon = rootView.findViewById(R.id.big_icon)
        sendButton = rootView.findViewById(R.id.send_button)
        mediaContext.requestParams.callerType
        val buttonTextId = when (mediaContext.requestParams.callerType) {
            MediaPickerHelper.PICKER_CALLER_TYPE.CHAT -> R.string.gallery_send
            else -> R.string.gallery_confirm_ok
        }
        sendButton.setText(buttonTextId)
        sendButton.setOnClickListener { save({ path -> sendToChatroom(path) }, getSaveFileName()) }
        initBottomButton()
        return rootView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        gifPreview.post {
            gifPreview.apply {
                setBackgroundMultiFrame(imagesPaths, frameDelayMsec)
                setAspectRatio(if (isFromCamera) cameraRatio else ratio.ratio)
                setBackgroundMultiFrameAlignment(ratio.alignment)
                setBackgroundPlayDirection(direction.direction)
                speedRatio = speed.speed
            }
        }
        super.onViewCreated(view, savedInstanceState)
    }

    override fun onStop() {
        isSaveCancel = true
        dismissProgressDialog()
        super.onStop()
    }

    override fun onRelease() {
    }

    private fun initFields() {
        arguments?.let {
            mode = MediaPickerInfo.MODE.valueOf(it.getString(ARG_PICKER_MODE))
            imagesPaths = it.getStringArrayList(ARG_MEDIA_ITEMS)
            frameDelayMsec = it.getInt(ARG_FRAME_DELAY_MSEC)
            isFromCamera = it.getBoolean(ARG_IS_FROM_CAMERA)
            cameraRatio = it.getDouble(ARG_CAMERA_RATIO)
        }
    }

    private fun initBottomButton() {
        bottomButtonView.addView(speedButton)
        bottomButtonView.addView(if (isFromCamera) directionButton else ratioButton)
        bottomButtonView.addView(saveButton)
    }

    private fun changeSpeed() {
        speed = speed.next()
        showBigIconImage(speed.bigIcon)
        speedButton.setText(speed.description)
        gifPreview.speedRatio = speed.speed
    }

    private fun changeDirection() {
        direction = direction.next()
        showBigIconImage(direction.bigIcon)
        gifPreview.setBackgroundPlayDirection(direction.direction)
        directionButton.apply {
            setIcon(direction.iconRes)
            setText(direction.textRes)
        }
    }

    private fun changeAspectRatio() {
        ratio = ratio.next()
        showBigIconImage(ratio.bigIcon)
        gifPreview.apply {
            setBackgroundMultiFrameAlignment(ratio.alignment)
            setAspectRatio(ratio.ratio)
        }
        ratioButton.apply {
            setIcon(ratio.iconRes)
            setText(ratio.textRes)
        }
    }

    private fun showProgressDialog(dialogStringRes: Int) = activity?.let {
        LineProgressDialog(activity).apply {
            setMessage(getString(dialogStringRes))
            setProgressStyle(LineProgressDialog.STYLE_HORIZONTAL)
            max = 100
            setCancelable(false)
            show()
        }
    }

    private fun dismissProgressDialog() {
        progressDialog?.run {
            if (isShowing) {
                dismiss()
            }
        }
    }

    private fun save(saveFun: (path: String) -> Unit, fileName: String) {
        if (StorageUtils.getAvailableExternalStorageSpace() < MIN_AVAILABLE_STORAGE_BYTE) {
            LineDialogHelper.showCapacityShortageExtStorageDialog(context, null)
            return
        }

        isSaveCancel = false
        progressDialog = showProgressDialog(R.string.gallery_creategif_label_processing)
        errorDialog = LineDialogHelper.createConfirmDialog(context, R.string.gallery_gif_error_temporary, null)

        MeiSDK.createImageCompositor().apply {
            setMeiCanvasView(gifPreview)
            setEventListener(SaveEventListener(saveFun, progressDialog, errorDialog))
            setOutputWidth(if (isFromCamera) 450 else 960)
            setSavedFilePath(fileName)
            composite()
        }
    }

    private fun getSaveFileName() = ContentsUtils.getFilePathWithSystemCurrentTimeMillis() + ".gif"

    private fun getKeepFileName() =
            LineMediaFileManager.getGifMakerCacheDirPath() + "/LINE_P" +
                    SimpleDateFormat("yyyyMMdd_HHmmssSS").format(Date(System.currentTimeMillis())) + ".gif"

    private fun save() {
        ToastHelper.show(activity?.getString(R.string.gallery_saved))

        sendGAEVentForSaveOrSend(if (isFromCamera) MediaGAEvents.MEDIA_CAMERA_GIF_EDIT_SAVE else MediaGAEvents.MEDIA_GIF_MAKER_SAVE)
    }

    private fun sendToChatroom(path: String) {
        activity?.let {
            val item = PickerMediaItem()
            item.filePath = path
            item.fileSize = File(path).length()
            item.source = MediaItem.Source.GIF_MAKER
            fragmentSubject.onNext(MediaFragmentParam.Action.GIF_MAKER_CLICK_SEND, item)
            it.requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_SENSOR
            sendGAEVentForSaveOrSend(if (isFromCamera) MediaGAEvents.MEDIA_CAMERA_GIF_EDIT_SEND else MediaGAEvents.MEDIA_GIF_MAKER_SEND)
        }
    }

    private fun sendToKeep(path: String) {
        activity?.let {
            val extension = MimeTypeMap.getFileExtensionFromUrl(path)
            val mimeType = MimeTypeMap.getSingleton().getMimeTypeFromExtension(extension)
            LineAccessForPickerHelper.saveItemToKeep(it, path, mimeType)
            sendGAEVentForSaveOrSend(if (isFromCamera) MediaGAEvents.MEDIA_CAMERA_GIF_EDIT_KEEP else MediaGAEvents.MEDIA_GIF_MAKER_KEEP)
        }
    }

    private fun showBigIconImage(imgRes: Int) {
        val animation = AlphaAnimation(1.0f, 0.0f)
        animation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationRepeat(animation: Animation?) {
            }

            override fun onAnimationEnd(animation: Animation?) {
                bigIcon.visibility = View.GONE
            }

            override fun onAnimationStart(animation: Animation?) {
                bigIcon.visibility = View.VISIBLE
            }
        })
        animation.duration = 1000

        bigIcon.clearAnimation()
        bigIcon.setImageResource(imgRes)
        bigIcon.startAnimation(animation)
    }

    private fun sendGAEVentForSaveOrSend(gaEvent: MediaGAEvents) {
        val customDimensions = getGaCustomDimensions()
        LineAccessForCommonHelper.getLineAccess().sendGaEvent(
                gaEvent.category, gaEvent.action, gaEvent.label, customDimensions, null)
    }

    private fun getGaCustomDimensions(): GACustomDimensions {
        val customDimensions = GACustomDimensions()

        if (isFromCamera) {
            customDimensions.add(MediaGACustomDimensionType.MEDIA_LOCATION.dimensionNumber,
                    mediaContext.requestParams.callerType.gaValue)
            customDimensions.add(MediaGACustomDimensionType.CREATE_GIF_SPEED.dimensionNumber,
                    if (direction.direction == PlayDirection.FORWARD) speed.description else speed.description + "_backward")
        } else {
            customDimensions.add(MediaGACustomDimensionType.CREATE_GIF_SPEED.dimensionNumber, speed.description)
            customDimensions.add(MediaGACustomDimensionType.CREATE_GIF_RATIO.dimensionNumber, ratio.toString().toLowerCase())
        }
        return customDimensions
    }
}
