package com.linecorp.line.media.picker.mode.gif

import android.os.Bundle
import android.support.v4.app.Fragment
import com.linecorp.line.common.PickerMediaItem
import com.linecorp.line.media.picker.MediaPickerHelper
import com.linecorp.line.media.picker.MediaPickerInfo
import com.linecorp.line.media.picker.action.OnFragmentAction
import com.linecorp.line.media.picker.action.impl.GifMakerFragmentActionImpl
import com.linecorp.line.media.picker.base.MediaContextScreen
import com.linecorp.line.media.picker.embedded.MediaContextFragmentViewHolder
import com.linecorp.line.media.picker.embedded.facade.LineGifMakerFacade
import com.linecorp.line.media.picker.embedded.facade.listener.GifMakerFacadeListener
import com.linecorp.line.media.picker.fragment.gif.GIFMakersFragment
import com.linecorp.line.media.picker.fragment.main.MediaPickerBaseFragment
import com.linecorp.line.media.picker.mode.MediaContextController
import com.linecorp.line.media.picker.permission.PermissionChecker
import com.linecorp.line.media.picker.subjects.FragmentSubscriber
import com.linecorp.line.media.util.MediaPickerUtils
import jp.naver.line.androie.common.CommonBaseFragmentActivity

class LineGifMaker : MediaContextController<LineGifMakerFacade> {
    private val gifMakerFacade: LineGifMakerFacade by lazy {
        LineGifMakerFacade(this)
    }

    private var gifMakerFragment: GIFMakersFragment? = null
    private val subscriberMap: MutableMap<Fragment, FragmentSubscriber> = HashMap()
    private val isFromCamera: Boolean

    constructor(mediaContextId: Int,
                activity: CommonBaseFragmentActivity,
                requestParams: MediaPickerHelper.MediaPickerParams,
                fragmentViewHolder: MediaContextFragmentViewHolder, pickerItems: List<PickerMediaItem>,
                frameDelay: Int,
                isFromCamera: Boolean,
                cameraRatio: Double) :
            super(mediaContextId, activity, requestParams, fragmentViewHolder) {
        this.isFromCamera = isFromCamera
        gifMakerFragment = GIFMakersFragment.newInstance(MediaPickerInfo.MODE.GIF_MAKER_IMAGE, pickerItems as ArrayList, frameDelay, isFromCamera, cameraRatio)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        attachFragment(gifMakerFragment as GIFMakersFragment, fragmentViewHolder.gifMakerFragmentContainer, MediaPickerUtils.getGifMakerFragmentTag(mediaContext.id))
    }

    override fun onFragmentDetached(fragment: Fragment) {
        super.onFragmentDetached(fragment)
        gifMakerFragment = null
        unsubscribeFragment(fragment)
    }

    override fun onFragmentPreAttached(fragment: Fragment) {
        super.onFragmentPreAttached(fragment)
        subscribeFragment(fragment, GifMakerFragmentActionImpl(mediaContext))
    }

    override fun detachFragments() {
        detachFragment(MediaPickerUtils.getGifMakerFragmentTag(mediaContext.id))
        gifMakerFragment = null
    }

    override fun getFacade(): LineGifMakerFacade = gifMakerFacade

    override fun getPermissions(): Array<String> = PermissionChecker.PERMISSION_STORAGE

    override fun onBackPressed(): Boolean {
        val consumed = super.onBackPressed()
        if (consumed) {
            return true
        }

        if (mediaContext.facadeListener is GifMakerFacadeListener) {
            val listener = mediaContext.facadeListener as GifMakerFacadeListener
            listener.onGifMakerCanceled()
            return true
        }
        return false
    }

    override fun getCurrentScreen(): MediaContextScreen? {
        val screen = super.getCurrentScreen()
        if (screen != null) {
            return screen
        }

        if (gifMakerFragment != null) {
            if (isFromCamera) {
                return MediaContextScreen.GIF_MAKER_EDIT
            }
            return MediaContextScreen.GIF_MAKER
        }
        return null
    }

    private fun unsubscribeFragment(fragment: Fragment) {
        if (fragment !is MediaPickerBaseFragment) {
            return
        }
        val subscriber = subscriberMap.remove(fragment)
        subscriber?.run {
            unsubscribe()
        }
    }

    private fun subscribeFragment(fragment: Fragment, onFragmentAction: OnFragmentAction) {
        if (fragment !is MediaPickerBaseFragment) {
            return
        }
        var subscriber: FragmentSubscriber? = subscriberMap[fragment]
        if (subscriber != null) {
            return
        }

        subscriber = FragmentSubscriber(fragment.fragmentSubject, onFragmentAction)
        subscriber.subscribe()
        subscriberMap.put(fragment, subscriber)
    }
}