package com.linecorp.line.media.picker.mode.gif

import android.app.Activity
import com.linecorp.line.camera.util.CameraSettingManager
import com.linecorp.line.common.PickerMediaItem
import com.linecorp.line.media.R
import com.linecorp.line.media.picker.MediaPickerHelper
import com.linecorp.line.media.picker.MediaPickerInfo
import com.linecorp.line.media.picker.base.MediaContext
import com.linecorp.line.media.picker.embedded.MediaContextAttachRequest
import com.linecorp.line.media.picker.embedded.MediaContextFragmentViewHolder
import com.linecorp.line.media.picker.embedded.facade.LineGifMakerFacade
import com.linecorp.line.media.picker.embedded.facade.MediaFacade
import com.linecorp.line.media.picker.embedded.facade.listener.MediaFacadeListener
import com.linecorp.line.media.picker.permission.PermissionGrantedAction
import com.linecorp.yuki.camera.effect.android.util.CameraConst
import jp.naver.gallery.android.media.MediaItem
import jp.naver.line.androie.common.CommonBaseFragmentActivity
import jp.naver.line.androie.common.dialog.LineDialogHelper

class MediaGifMakerHelper {
    companion object {
        const val DEFAULT_DELAY = 500
    }

    @JvmOverloads
    fun startGifMaker(activity: CommonBaseFragmentActivity,
                      mediaContext: MediaContext,
                      gifMakerFacadeListenerImpl: MediaFacadeListener<MediaFacade>,
                      items: List<PickerMediaItem>? = null,
                      frameDelay: Int = DEFAULT_DELAY,
                      isFromCamera: Boolean = false): LineGifMakerFacade? {
        val pickerRequestParam = mediaContext.requestParams
        val param = MediaPickerHelper.createGifMakerBuilder(activity, mediaContext.requestParams.callerType)
                .setMaxGifMakerMediaCount(pickerRequestParam.gifMakerMediaCount, pickerRequestParam.gifMakerMediaCountWarning)
                .setSupportAgifExtension(pickerRequestParam.isSupportAgifExtension)
                .buildParams()

        val viewHolder = MediaContextFragmentViewHolder()
        val request = MediaContextAttachRequest(
                param,
                activity,
                viewHolder,
                PermissionGrantedAction.ATTACH, gifMakerFacadeListenerImpl)

        val gifContextId = Integer.MIN_VALUE + mediaContext.id
        val gifMaker = LineGifMaker(gifContextId, activity, param, viewHolder,
                items ?: mediaContext.itemManager.selectedItems.values.toList(),
                frameDelay, isFromCamera, getCameraRatio(activity))

        return gifMaker.attach(request)
    }

    fun canStartGifMaker(activity: CommonBaseFragmentActivity, mediaContext: MediaContext): Boolean {
        return !checkContainsIllegalTypeItemsToStartGifMaker(activity, mediaContext.itemManager.selectedItems.values) &&
                !checkIllegalMaxItemCountToStartGifMaker(mediaContext, activity)
    }

    private fun checkIllegalMaxItemCountToStartGifMaker(
            mediaContext: MediaContext, activity: Activity): Boolean {

        if (mediaContext.itemManager.selectedItems.size > mediaContext.requestParams.gifMakerMediaCount) {
            LineDialogHelper.showConfirmDialog(activity, mediaContext.requestParams.gifMakerMediaCountWarning, null)
            return true
        }
        return false
    }

    private fun checkContainsIllegalTypeItemsToStartGifMaker(activity: Activity, selectedItems: Collection<PickerMediaItem>): Boolean {
        for (item in selectedItems) {
            if (MediaItem.MEDIA_TYPE_IMAGE != item.mediaType
                    || item.isExtensionTypeGif) {
                LineDialogHelper.showConfirmDialog(activity,
                        activity.getString(R.string.gallery_creategif_onlyphoto), null)
                return true
            }

            val dimensionPair = item.getDimension()
            val isBigSizeImage = item.getFileSize() >= MediaPickerInfo.MAX_FILE_SIZE_GIF_MAKER_IMAGE
            val isOverPixelSize = dimensionPair.first * dimensionPair.second >= MediaPickerInfo.MAX_PIXEL_SIZE_GIF_MAKER_IMAGE
            if (isBigSizeImage || isOverPixelSize) {
                LineDialogHelper.showConfirmDialog(activity,
                        activity.getString(R.string.gallery_creategif_image_size_error), null)
                return true
            }
        }
        return false
    }

    private fun getCameraRatio(activity: Activity): Double {
        val cameraSettingManager = CameraSettingManager(activity)

        when (cameraSettingManager.ratio) {
            CameraConst.Ratio.RATIO_16x9 -> return (9 / 16).toDouble()
            CameraConst.Ratio.RATIO_9x16 -> return (16 / 9).toDouble()
            CameraConst.Ratio.RATIO_4x3 -> return (3 / 4).toDouble()
            CameraConst.Ratio.RATIO_3x4 -> return (4 / 3).toDouble()
            else -> return 1.0
        }
    }
}
