package jp.naver.gallery.android.adapter

import android.annotation.SuppressLint
import android.os.Parcelable
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v4.app.FragmentTransaction
import android.support.v4.view.PagerAdapter
import android.view.View
import android.view.ViewGroup

@SuppressLint("LongLogTag")
abstract class RemovableFragmentPagerAdapter(protected val mFragmentManager: FragmentManager) : PagerAdapter() {
    protected var mCurTransaction: FragmentTransaction? = null

    abstract fun getItem(position: Int): Fragment

    override fun startUpdate(container: ViewGroup) {}

    override fun instantiateItem(container: ViewGroup, position: Int): Any {
        mCurTransaction = mCurTransaction ?: mFragmentManager.beginTransaction()

        val name = makeFragmentName(container.id, position)
        var fragment: Fragment? = mFragmentManager.findFragmentByTag(name)
        if (fragment != null) {
            mCurTransaction!!.attach(fragment)
        } else {
            fragment = getItem(position)
            mCurTransaction!!.add(container.id, fragment, makeFragmentName(container.id, position))
        }

        return fragment
    }

    override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
        mCurTransaction = mCurTransaction ?: mFragmentManager.beginTransaction()
        mCurTransaction?.detach(`object` as Fragment?)
    }

    override fun finishUpdate(container: ViewGroup) {
        mCurTransaction?.let {
            it.commitAllowingStateLoss()
            mCurTransaction = null
            mFragmentManager.executePendingTransactions()
        }
    }

    override fun isViewFromObject(view: View, `object`: Any): Boolean {
        return (`object` as Fragment).view === view
    }

    override fun saveState(): Parcelable? = null

    override fun restoreState(state: Parcelable?, loader: ClassLoader?) {}

    /**
     * If you call the notifyDataSetChanged () method on the * adapter, the pager will have all of
     * its child pages (old, current, and the next three children in common).
     * If the following method is called, returning that the position of the object (page) has
     * changed (POSITION_NONE, meaning deleted here)
     * The getItem () method is called (a new fragment is created) and the final pager's screen is updated.
     */
    override fun getItemPosition(`object`: Any): Int {
        val tag = (`object` as Fragment).tag

        val fragment = mFragmentManager.findFragmentByTag(tag)
        return if (fragment == null) {
            PagerAdapter.POSITION_NONE
        } else {
            PagerAdapter.POSITION_UNCHANGED
        }
    }

    /**
     * Deletes the fragment at the given index.
     * Here we delete all fragment instances after the given index.
     * Originally, it is best to reduce the index value by one from the tag name of the fragment
     * corresponding to the index, * but if you change the tag name, an exception occurs.
     *
     * @param container
     * @param position
     */
    fun removeItem(container: View, position: Int) {
        val fragmentTransaction = mFragmentManager.beginTransaction()

        (position until count)
                .asSequence()
                .map { makeFragmentName(container.id, it) }
                .mapNotNull { mFragmentManager.findFragmentByTag(it) }
                .forEach { fragmentTransaction.remove(it) }

        fragmentTransaction.commitAllowingStateLoss()

        mFragmentManager.executePendingTransactions()
    }

    companion object {
        fun makeFragmentName(viewId: Int, index: Int) = "photoAlbum:switcher:$viewId:$index"
    }
}
