package jp.naver.line.androie.common.lib.util

import android.support.v4.util.PatternsCompat
import android.text.util.Linkify
import android.text.util.Linkify.MatchFilter
import android.text.util.Linkify.TransformFilter
import java.util.regex.Matcher
import java.util.regex.Pattern

// TODO: remove this class and move data class + static functions in a kotlin file.
// This class is preserved only because line-keep branch state is currently unclear.
class LinkifyUtil private constructor() {

    data class LinkSpec(
            @JvmField val url: String,
            @JvmField val start: Int,
            @JvmField val end: Int
    )

    companion object {
        /** Workaround for LINEAND-20834 */
        const val LINK_LENGTH_MAXIMUM = 2083

        // TODO: use Kotlin sequence for optimized list concatenation and return immutable collections

        @JvmStatic
        fun gatherWebUrlLinks(links: MutableList<LinkSpec>, text: CharSequence) {
            @Suppress("DEPRECATION")
            gatherLinks(links, text, LinePatterns.NHNCORP_APPS_PATTERN, AppSchemes.NHNCORP_APP_SCHEMES)
            gatherLinks(links, text, LinePatterns.LINE_PATTERN, arrayOf("line://", "lineb://"))
            gatherLinks(links, text, LinePatterns.LINE_FAMILY_APPS_PATTERN, AppSchemes.LineFamilyApps.getSchemes())
            gatherLinks(links, text, PatternsCompat.AUTOLINK_WEB_URL,
                    arrayOf("http://", "https://", "rtsp://"),
                    Linkify.sUrlMatchFilter, null)
        }

        @JvmStatic
        fun gatherEmailLinks(links: MutableList<LinkSpec>, text: CharSequence) =
                gatherLinks(links, text, PatternsCompat.EMAIL_ADDRESS, arrayOf("mailto:"))

        /** @see android.util.Patterns.PHONE */
        @JvmStatic
        fun gatherPhoneLinks(links: MutableList<LinkSpec>, text: CharSequence) =
                gatherLinks(links, text, android.util.Patterns.PHONE, arrayOf("tel:"),
                        Linkify.sPhoneNumberMatchFilter, Linkify.sPhoneNumberTransformFilter)

        @JvmStatic
        fun gatherLinks(
                links: MutableList<LinkSpec>,
                text: CharSequence,
                pattern: Pattern,
                schemes: Array<String>,
                matchFilter: MatchFilter? = null,
                transformFilter: TransformFilter? = null
        ) {
            var indexOffset = 0
            text.toString()
                    .split(Regex("\\s"))
                    .forEach { token ->
                        links.addAll(gatherLinksInToken(token, pattern, schemes, matchFilter, transformFilter, indexOffset))
                        // Update offset after this token + separation character (whitespace char length = 1)
                        indexOffset += token.length + 1
                    }
        }

        /**
         * Workaround for LINEAND-19622
         * TODO: remove after https://issuetracker.google.com/issues/65001273 is resolved
         */
        private fun gatherLinksInToken(
                token: String,
                pattern: Pattern,
                schemes: Array<String>,
                matchFilter: MatchFilter?,
                transformFilter: TransformFilter?,
                indexOffset: Int
        ): List<LinkSpec> {
            if (token.length > LINK_LENGTH_MAXIMUM) {
                return emptyList()
            }

            val links = ArrayList<LinkSpec>(1)
            val m = pattern.matcher(token)

            while (m.find()) {
                val start = m.start()
                val end = m.end()

                if (matchFilter == null || matchFilter.acceptMatch(token, start, end)) {
                    links.add(LinkSpec(
                            makeUrl(m, schemes, transformFilter),
                            start + indexOffset,
                            end + indexOffset
                    ))
                }
            }

            return links
        }

        @JvmStatic
        private fun makeUrl(
                m: Matcher,
                prefixes: Array<String>,
                filter: TransformFilter?
        ): String {
            var url = m.group(0)
            if (filter != null) {
                url = filter.transformUrl(m, url)
            }

            var hasPrefix = false

            for (i in prefixes.indices) {
                if (url.regionMatches(0, prefixes[i], 0,
                        prefixes[i].length, ignoreCase = true)) {
                    hasPrefix = true

                    // Fix capitalization if necessary
                    if (!url.regionMatches(0, prefixes[i], 0,
                            prefixes[i].length, ignoreCase = false)) {
                        url = prefixes[i] + url.substring(prefixes[i].length)
                    }

                    break
                }
            }

            if (!hasPrefix) {
                url = prefixes[0] + url
            }

            return url
        }
    }
}
