package jp.naver.line.androie.common.lib.util

import android.app.ActivityManager
import android.content.Context

/**
 * Collection of utility methods to get device memory info.
 */
object MemoryInfoUtils {
    private const val AUTO_PLAYABLE_GIF_RAM_THRESHOLD: Long = 1024 * 1024 * 1024 // 1GiB

    /**
     * Returns true if the device has enough ram to play animation gif
     * and the system doesn't consider itself currently to be in low memory situation.
     */
    @JvmStatic
    fun hasEnoughTotalMemoryForAnimationGif(context: Context): Boolean {
        val memoryInfo = getMemoryInfo(context)
        return memoryInfo.totalMem >= AUTO_PLAYABLE_GIF_RAM_THRESHOLD && !memoryInfo.lowMemory
    }

    /**
     * Returns true if the device has enough ram to play animation gif.
     */
    @JvmStatic
    fun hasEnoughRamForAnimationGif(context: Context): Boolean {
        val memoryInfo = getMemoryInfo(context)
        return memoryInfo.totalMem >= AUTO_PLAYABLE_GIF_RAM_THRESHOLD
    }

    private fun getMemoryInfo(context: Context): ActivityManager.MemoryInfo {
        val activityManager = context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        return ActivityManager.MemoryInfo().also { activityManager.getMemoryInfo(it) }
    }
}
