package jp.naver.line.androie.common.theme

import android.util.SparseArray
import jp.naver.android.common.R

private typealias ViewThemeMap = SparseArray<ThemeElementMappingData>

object LineCommonThemeMapper {
    @JvmStatic
    fun getThemeMappingData(themeKey: ThemeKey): ViewThemeMap? = when (themeKey) {
        ThemeKey.VIEW_COMMON -> createViewThemeMap(
                R.id.view_common to
                        ThemeElementMappingData.Builder(R.id.view_common)
                                .addElementKey(*CommonThemeKeys.Common.ROOT)
                                .build())

        ThemeKey.MAIN_VIEW_COMMON -> createViewThemeMap(
                R.id.view_common to
                        ThemeElementMappingData.Builder(R.id.view_common)
                                .addElementKey(*CommonThemeKeys.MainTab.ROOT)
                                .build())

        ThemeKey.NAVIGATION_BAR -> createViewThemeMap(
                R.id.header_title to
                        ThemeElementMappingData.Builder(R.id.header_title)
                                .addElementKey(*CommonThemeKeys.Header.TITLE)
                                .addMappingTarget(ThemeElementValueType.TEXT)
                                .build(),
                R.id.header_left_button_layout to
                        ThemeElementMappingData.Builder(R.id.header_left_button_layout)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.header_left_button_text to
                        ThemeElementMappingData.Builder(R.id.header_left_button_text)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.TEXT)
                                .build(),
                R.id.header_left_button_img to
                        ThemeElementMappingData.Builder(R.id.header_left_button_img)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_left_button_live to
                        ThemeElementMappingData.Builder(R.id.header_left_button_img)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_right_button_layout to
                        ThemeElementMappingData.Builder(R.id.header_right_button_layout)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.header_right_button_text to
                        ThemeElementMappingData.Builder(R.id.header_right_button_text)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.TEXT)
                                .build(),
                R.id.header_right_button_img to
                        ThemeElementMappingData.Builder(R.id.header_right_button_img)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_root to
                        ThemeElementMappingData.Builder(R.id.header_root)
                                .addElementKey(*CommonThemeKeys.Header.ROOT)
                                .build(),
                R.id.header_title_left_image to
                        ThemeElementMappingData.Builder(R.id.header_title_left_image)
                                .addElementKey(*CommonThemeKeys.Header.TITLE)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_title_count to
                        ThemeElementMappingData.Builder(R.id.header_title_count)
                                .addElementKey(*CommonThemeKeys.Header.TITLE)
                                .addMappingTarget(ThemeElementValueType.TEXT)
                                .build(),
                R.id.header_more_icon to
                        ThemeElementMappingData.Builder(R.id.header_more_icon)
                                .addElementKey(*CommonThemeKeys.Header.TITLE)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_mute_icon to
                        ThemeElementMappingData.Builder(R.id.header_mute_icon)
                                .addElementKey(*CommonThemeKeys.Header.TITLE)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_left_noti_new to
                        ThemeElementMappingData.Builder(R.id.header_left_noti_new)
                                .addElementKey(*CommonThemeKeys.Header.NEW_DOT_ICON)
                                .build(),
                R.id.header_left_noti_count to
                        ThemeElementMappingData.Builder(R.id.header_left_noti_count)
                                .addElementKey(*CommonThemeKeys.Header.COUNT_BADGE)
                                .build(),
                R.id.header_middle_button_layout to
                        ThemeElementMappingData.Builder(R.id.header_middle_button_layout)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.header_middle_button_text to
                        ThemeElementMappingData.Builder(R.id.header_middle_button_text)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.TEXT)
                                .build(),
                R.id.header_middle_button_img to
                        ThemeElementMappingData.Builder(R.id.header_middle_button_img)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_middle_noti_new to
                        ThemeElementMappingData.Builder(R.id.header_middle_noti_new)
                                .addElementKey(*CommonThemeKeys.Header.NEW_DOT_ICON)
                                .build(),
                R.id.header_middle_noti_count to
                        ThemeElementMappingData.Builder(R.id.header_middle_noti_count)
                                .addElementKey(*CommonThemeKeys.Header.COUNT_BADGE)
                                .build(),
                R.id.header_right_noti_new to
                        ThemeElementMappingData.Builder(R.id.header_right_noti_new)
                                .addElementKey(*CommonThemeKeys.Header.NEW_DOT_ICON)
                                .build(),
                R.id.searchbar_back_button_bg to
                        ThemeElementMappingData.Builder(R.id.searchbar_back_button_bg)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.header_up_button to
                        ThemeElementMappingData.Builder(R.id.header_up_button)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.header_up_button_bg to
                        ThemeElementMappingData.Builder(R.id.header_up_button_bg)
                                .addElementKey(*CommonThemeKeys.Header.BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build())

        ThemeKey.NAVIGATION_BAR_MORE_MENU -> createViewThemeMap(
                R.id.common_popup_item_background to
                        ThemeElementMappingData.Builder(R.id.common_popup_item_background)
                                .addElementKey(*CommonThemeKeys.Header.MENU)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.common_popup_item_title to
                        ThemeElementMappingData.Builder(R.id.common_popup_item_title)
                                .addElementKey(*CommonThemeKeys.Header.MENU)
                                .addMappingTarget(ThemeElementValueType.TEXT)
                                .build())

        ThemeKey.NAVIGATION_BAR_SEARCH -> createViewThemeMap(
                R.id.v2_common_searchbar_bg to
                        ThemeElementMappingData.Builder(R.id.v2_common_searchbar_bg)
                                .addElementKey(*CommonThemeKeys.SearchBar.ROOT)
                                .build(),
                R.id.searchbar_back_button to
                        ThemeElementMappingData.Builder(R.id.searchbar_back_button)
                                .addElementKey(*CommonThemeKeys.SearchBar.BACK_ICON)
                                .build(),
                R.id.searchbar_cancel_button to
                        ThemeElementMappingData.Builder(R.id.searchbar_cancel_button)
                                .addElementKey(*CommonThemeKeys.SearchBar.CANCEL_ICON)
                                .build(),
                R.id.searchbar_input_text to
                        ThemeElementMappingData.Builder(R.id.searchbar_input_text)
                                .addElementKey(*CommonThemeKeys.SearchBar.TEXT)
                                .build(),
                R.id.searchbar_div to
                        ThemeElementMappingData.Builder(R.id.searchbar_div)
                                .addElementKey(*CommonThemeKeys.SearchBar.INPUT_FIELD_UNDER_LINE)
                                .build())

        ThemeKey.PASSCOCE -> createViewThemeMap(
                R.id.thk_passcode_input1_on to
                        ThemeElementMappingData.Builder(R.id.thk_passcode_input1_on)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE1_ICON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.thk_passcode_input2_on to
                        ThemeElementMappingData.Builder(R.id.thk_passcode_input2_on)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE2_ICON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.thk_passcode_input3_on to
                        ThemeElementMappingData.Builder(R.id.thk_passcode_input3_on)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE3_ICON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.thk_passcode_input4_on to
                        ThemeElementMappingData.Builder(R.id.thk_passcode_input4_on)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE4_ICON)
                                .addMappingTarget(ThemeElementValueType.IMAGE)
                                .build(),
                R.id.passcode_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.ROOT)
                                .build(),
                R.id.passcode_top_desc to
                        ThemeElementMappingData.Builder(R.id.passcode_top_desc)
                                .addElementKey(*CommonThemeKeys.Passcode.TOP)
                                .build(),
                R.id.passcode_prompt to
                        ThemeElementMappingData.Builder(R.id.passcode_prompt)
                                .addElementKey(*CommonThemeKeys.Passcode.TITLE)
                                .build(),
                R.id.passcode_desc to
                        ThemeElementMappingData.Builder(R.id.passcode_desc)
                                .addElementKey(*CommonThemeKeys.Passcode.DESCRIPTION)
                                .build(),
                R.id.passcode_pass to
                        ThemeElementMappingData.Builder(R.id.passcode_pass)
                                .addElementKey(*CommonThemeKeys.Passcode.TOP)
                                .build(),
                R.id.passcode_input1 to
                        ThemeElementMappingData.Builder(R.id.passcode_input1)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE1_ICON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_input2 to
                        ThemeElementMappingData.Builder(R.id.passcode_input2)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE2_ICON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_input3 to
                        ThemeElementMappingData.Builder(R.id.passcode_input3)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE3_ICON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_input4 to
                        ThemeElementMappingData.Builder(R.id.passcode_input4)
                                .addElementKey(*CommonThemeKeys.Passcode.PASSCODE4_ICON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn1_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn1_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon1 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon1)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn2_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn2_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon2 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon2)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn3_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn3_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon3 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon3)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn4_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn4_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon4 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon4)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn5_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn5_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon5 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon5)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn6_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn6_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon6 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon6)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn7_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn7_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon7 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon7)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn8_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn8_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon8 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon8)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn9_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn9_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon9 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon9)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn_cancel to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_cancel)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_CANCEL)
                                .build(),
                R.id.passcode_btn0_bg to
                        ThemeElementMappingData.Builder(R.id.passcode_btn0_bg)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON)
                                .addMappingTarget(ThemeElementValueType.BACKGROUND)
                                .build(),
                R.id.passcode_btn_icon0 to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon0)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_LABEL)
                                .build(),
                R.id.passcode_btn_icon_del to
                        ThemeElementMappingData.Builder(R.id.passcode_btn_icon_del)
                                .addElementKey(*CommonThemeKeys.Passcode.KEYBOARD_BUTTON_DELETE)
                                .build())
        else -> null
    }

    private fun createViewThemeMap(vararg elements: Pair<Int, ThemeElementMappingData>) =
            ViewThemeMap().also {
                for ((resourceId, mappingData) in elements) {
                    it.put(resourceId, mappingData)
                }
            }
}
