package jp.naver.line.androie.common.theme

/**
 * Types of theme json key suffix, such as image, background, or tintcolor.
 */
enum class ThemeElementJsonKeyType(
        private val suffix: String,
        private val shouldContainBackground: Boolean
) {
    // The following enumeration order is important because json key pattern will be checked
    // according to the following order.
    // For example, "*.image" pattern includes "*.background.image".
    // TODO: Consider to change theme key rule to be simple.
    // "*.text.tintcolor" is parsed as IMAGE_TINT_COLOR. It is confusing because it is an image tint
    // color even if the key name includes "text".
    BACKGROUND_IMAGE_PATTERN(".image.pattern", true),
    IMAGE_PATTERN(".image.pattern", false),

    BACKGROUND_IMAGE(".image", true),
    IMAGE(".image", false),

    BACKGROUND_IMAGE_TINT_COLOR(".tintcolor", true),
    IMAGE_TINT_COLOR(".tintcolor", false),

    BACKGROUND_COLOR(".color", true),
    TEXT_COLOR(".color", false),

    TEXT_SHADOW_COLOR(".shadow", false),
    HINT_TEXT_COLOR(".hint", false),

    UNKNOWN("", false);

    class ParseResult(val type: ThemeElementJsonKeyType, val keyBody: String)

    companion object {

        private val BACKGROUND_PREFIX_SUBKEY = "background."
        private val BACKGROUND_INTERMEDIATE_SUBKEY = ".background."

        @JvmStatic
        fun parse(elementJsonKey: String): ParseResult {
            val isBackgroundKey = elementJsonKey.startsWith(BACKGROUND_PREFIX_SUBKEY)
                    || elementJsonKey.contains(BACKGROUND_INTERMEDIATE_SUBKEY)

            return values().firstOrNull { type ->
                elementJsonKey.endsWith(type.suffix) &&
                        (!type.shouldContainBackground || isBackgroundKey)
            }?.let { type ->
                val keyBody = elementJsonKey.substring(
                        0, elementJsonKey.length - type.suffix.length)
                ParseResult(type, keyBody)
            } ?: ParseResult(UNKNOWN, elementJsonKey)
        }
    }
}
