package jp.naver.line.androie.common.theme

import java.util.ArrayList
import java.util.Collections
import java.util.EnumSet

/**
 * Data class to represent mapping data of a theme element and resource id.
 */
class ThemeElementMappingData(
        val resourceId: Int,
        val mappingTargets: Set<ThemeElementValueType>,
        val defaultValue: ThemeElementDefaultValue?,
        val elementKeys: Array<ThemeElementKey>) {

    constructor(
            resourceId: Int,
            vararg elementKeysList: Array<ThemeElementKey>
    ): this(resourceId,
            mappingTargets = ALL_TARGETS,
            defaultValue = null,
            elementKeys = elementKeysList.flatten().toTypedArray())

    class Builder(private val resourceId: Int) {
        private val elementKeys: MutableList<ThemeElementKey> = ArrayList(1)
        private val mappingTargets: MutableSet<ThemeElementValueType> =
                EnumSet.noneOf(ThemeElementValueType::class.java)
        private var defaultValue: ThemeElementDefaultValue? = null

        fun addElementKey(componentKey: String, elementKey: String): Builder =
                addElementKey(ThemeElementKey(componentKey, elementKey))

        fun addElementKey(vararg elementKeys: ThemeElementKey): Builder =
                apply { Collections.addAll(this.elementKeys, *elementKeys) }

        fun addMappingTarget(vararg mappingTargets: ThemeElementValueType): Builder =
                apply { Collections.addAll(this.mappingTargets, *mappingTargets) }

        fun setDefaultValue(type: ThemeElementDefaultValue.ValueType, resourceId: Int): Builder =
                apply { setDefaultValue(ThemeElementDefaultValue(type, resourceId)) }

        fun setDefaultValue(defaultValue: ThemeElementDefaultValue?): Builder =
                apply { this.defaultValue = defaultValue }

        fun build(): ThemeElementMappingData = ThemeElementMappingData(
                resourceId,
                if (mappingTargets.isEmpty()) ALL_TARGETS else mappingTargets,
                defaultValue,
                elementKeys.toTypedArray())
    }

    companion object {
        val ALL_TARGETS: Set<ThemeElementValueType> =
                EnumSet.allOf(ThemeElementValueType::class.java)
    }
}
