package jp.naver.line.androie.common.theme

/**
 * Class to get [ThemeElementValue] corresponding to the given [ThemeElementKey].
 * If multiple [ThemeElementKey] is given, this returns a [ThemeElementValue] that the
 * each field is overwritten by subsequent mapped non null values.
 */
object ThemeElementValueMerger {

    // The vararg size is 1 in most cases. So, we don't use map(...).fold(...) for such cases.
    fun getMergedValue(
            themeData: Map<ThemeElementKey, ThemeElementValue>,
            vararg elementKeys: ThemeElementKey
    ): ThemeElementValue? = when (elementKeys.size) {
        0 -> null
        1 -> themeData[elementKeys[0]]?.emptyToNull()
        else -> elementKeys
                .map(themeData::get)
                .fold(null, this::mergeThemeElementValue)
                ?.emptyToNull()
    }

    private fun mergeThemeElementValue(
            first: ThemeElementValue?,
            second: ThemeElementValue?
    ): ThemeElementValue? = when {
        first == null -> second
        second == null -> first
        else -> ThemeElementValue(
                second.image ?: first.image,
                second.imageTintColor ?: first.imageTintColor,
                second.backgroundColor ?: first.backgroundColor,
                second.backgroundImage ?: first.backgroundImage,
                second.backgroundImageTintColor ?: first.backgroundImageTintColor,
                second.textColor ?: first.textColor,
                second.textShadowColor ?: first.textShadowColor,
                second.hintTextColor ?: first.hintTextColor)
    }
}
