package jp.naver.line.androie.common.util

import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.Drawable
import android.graphics.drawable.StateListDrawable
import android.support.annotation.ColorRes
import android.support.annotation.DrawableRes
import android.support.v4.content.ContextCompat
import android.util.StateSet.WILD_CARD
import jp.naver.line.androie.common.util.StateListDrawableStateConstants.STATE_DISABLED
import jp.naver.line.androie.common.util.StateListDrawableStateConstants.STATE_PRESSED
import jp.naver.line.androie.common.util.StateListDrawableStateConstants.STATE_SELECTED
import java.util.Locale

/**
 * A builder class to generate [StateListDrawable].
 *
 * ## Caution
 *
 * - [normal] is mandatory.
 */
class StateListDrawableBuilder {
    private var normal: Drawable? = null
    private var pressed: Drawable? = null
    private var disabled: Drawable? = null
    private var selected: Drawable? = null

    fun normal(drawable: Drawable): StateListDrawableBuilder = apply {
        normal = drawable
    }

    fun normalFromColorRes(
            context: Context,
            @ColorRes colorResId: Int
    ): StateListDrawableBuilder = normal(context.colorDrawable(colorResId))

    fun normalFromDrawableRes(
            context: Context,
            @DrawableRes drawableResId: Int
    ): StateListDrawableBuilder = normal(context.drawable(drawableResId))

    fun pressed(drawable: Drawable): StateListDrawableBuilder = apply {
        pressed = drawable
    }

    fun pressedFromColorRes(
            context: Context,
            @ColorRes colorResId: Int
    ): StateListDrawableBuilder = pressed(context.colorDrawable(colorResId))

    fun pressedFromDrawableRes(
            context: Context,
            @DrawableRes drawableResId: Int
    ): StateListDrawableBuilder = pressed(context.drawable(drawableResId))

    fun disabled(drawable: Drawable): StateListDrawableBuilder = apply {
        disabled = drawable
    }

    @Suppress("unused")
    fun disabledFromColorRes(
            context: Context,
            @ColorRes colorResId: Int
    ): StateListDrawableBuilder = disabled(context.colorDrawable(colorResId))

    @Suppress("unused")
    fun disabledFromDrawableRes(
            context: Context,
            @DrawableRes drawableResId: Int
    ): StateListDrawableBuilder = disabled(context.drawable(drawableResId))

    fun selected(drawable: Drawable): StateListDrawableBuilder = apply {
        selected = drawable
    }

    @Suppress("unused")
    fun selectedFromColorRes(
            context: Context,
            @ColorRes colorResId: Int
    ): StateListDrawableBuilder = selected(context.colorDrawable(colorResId))

    fun selectedFromDrawableRes(
            context: Context,
            @DrawableRes drawableResId: Int
    ): StateListDrawableBuilder = selected(context.drawable(drawableResId))

    /**
     * Returns a new [StateListDrawable] instance.
     *
     * @throws IllegalStateException If [normal] wasn't provided.
     */
    fun build(): StateListDrawable = stateListDrawableOf(
            STATE_SELECTED to selected,
            STATE_PRESSED to pressed,
            STATE_DISABLED to disabled,
            WILD_CARD to (normal ?: throw IllegalStateException(
                    """There's no given information for the "normal" state """))
    )

    private fun stateListDrawableOf(
            vararg stateToDrawable: Pair<IntArray, Drawable?>
    ): StateListDrawable = stateToDrawable.fold(StateListDrawable()) { slDrawable, (state, drawable: Drawable?) ->
        if (drawable != null) slDrawable.addState(state, drawable)
        slDrawable
    }

    private fun Context.colorDrawable(@ColorRes colorResId: Int): ColorDrawable =
            ColorDrawable(ContextCompat.getColor(this, colorResId))

    private fun Context.drawable(@DrawableRes drawableResId: Int): Drawable =
            ContextCompat.getDrawable(this, drawableResId)
                    ?: error("Invalid drawableResId: ${"0x08X".format(Locale.ROOT, drawableResId)}")
}
