package jp.naver.line.androie.common.view

import android.os.Build
import android.support.v4.view.ViewCompat
import android.support.v4.widget.ImageViewCompat
import android.widget.ImageView

/**
 * Utility method to fix a bug of AppCompat: Tinting with [ImageViewCompat.setImageTintList] or
 * [ViewCompat.setBackgroundTintList] does not work on Marshmallow devices if the color list has
 * non-default color.
 *
 * Because this is very hacky and specialized logic just to avoid AppCompat library's regression,
 * the utility method is wrapped by object instead of revealing it as a global function.
 */
object ImageViewTintWorkaroundUtil {
    /**
     * Enforces to update the tint state of drawables of image and background to avoid Marshmallow's
     * AppCompat tinting bug. [ImageView] with a [ColorSateList][android.content.res.ColorStateList]
     * should call this method in
     * [View.drawableStateChanged][android.view.View.drawableStateChanged].
     */
    @JvmStatic
    fun onDrawableStateChanged(imageView: ImageView) {
        if (Build.VERSION.SDK_INT != Build.VERSION_CODES.M) {
            return
        }
        ImageViewCompat.setImageTintList(imageView, ImageViewCompat.getImageTintList(imageView))
        ViewCompat.setBackgroundTintList(imageView, ViewCompat.getBackgroundTintList(imageView))
    }
}
