package jp.naver.line.androie.common.view.listview

import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.LinearLayout
import android.widget.PopupWindow
import jp.naver.android.common.R

private const val MIN_REDISPLAY_MILLIS: Long = 200

class PopupListViewWindowHelper constructor(val context: Context) {
    private var lastDismissedTimestamp: Long = -1
    private val popupListView = PopupListView(context)
    private val popupWindow = PopupWindow(context.resources.getDimensionPixelSize(R.dimen.option_menu_width),
            ViewGroup.LayoutParams.WRAP_CONTENT)
            .apply {
                contentView = LinearLayout(context).apply {
                    addView(popupListView)
                }
                isOutsideTouchable = true
                setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
                setOnDismissListener {
                    lastDismissedTimestamp = System.currentTimeMillis()
                }
            }

    fun addListViewItem(name: String) =
            popupListView.addItem(PopupListItem.INVISIBLE_ICON_RES_ID, name)

    fun setOnItemClickListener(listener: AdapterView.OnItemClickListener) =
            popupListView.setOnItemClickListener(listener)

    fun setDialogMargin(left: Int, top: Int, right: Int, bottom: Int) =
            popupWindow.contentView.setPadding(left, top, right, bottom)

    fun show(anchor: View, xOff: Int, yOff: Int): Boolean {
        if (System.currentTimeMillis() - lastDismissedTimestamp < MIN_REDISPLAY_MILLIS) {
            return false
        }

        popupWindow.showAsDropDown(anchor, xOff, yOff)
        lastDismissedTimestamp = System.currentTimeMillis()
        return true
    }

    fun dismiss() = popupWindow.dismiss()

    val isShowing: Boolean
        get() = popupWindow.isShowing
}
